@@ -263,7 +263,7 @@ class TestPythonBinding(unittest.TestCase):
                                  s_umatrix_in=z)
 
     self.assertEqual(output['s_umatrix_out'].shape[0], 100)
-    self.assertEqual(output['s_umatrix_out'].dtype, np.long)
+    self.assertEqual(output['s_umatrix_out'].dtype, np.dtype('intp'))
 
     for i in range(100):
       self.assertEqual(output['s_umatrix_out'][i, 0], x.iloc[i] * 2)
@@ -284,7 +284,7 @@ class TestPythonBinding(unittest.TestCase):
                                  copy_all_inputs=True)
 
     self.assertEqual(output['s_umatrix_out'].shape[0], 100)
-    self.assertEqual(output['s_umatrix_out'].dtype, np.long)
+    self.assertEqual(output['s_umatrix_out'].dtype, np.dtype('intp'))
 
     for i in range(100):
       self.assertEqual(output['s_umatrix_out'][i, 0], x.iloc[i] * 2)
@@ -460,7 +460,7 @@ class TestPythonBinding(unittest.TestCase):
 
     self.assertEqual(output['umatrix_out'].shape[0], 100)
     self.assertEqual(output['umatrix_out'].shape[1], 4)
-    self.assertEqual(output['umatrix_out'].dtype, np.long)
+    self.assertEqual(output['umatrix_out'].dtype, np.dtype('intp'))
     for i in [0, 1, 3]:
       for j in range(100):
         self.assertEqual(x[j, i], output['umatrix_out'][j, i])
@@ -484,7 +484,7 @@ class TestPythonBinding(unittest.TestCase):
 
     self.assertEqual(output['umatrix_out'].shape[0], 100)
     self.assertEqual(output['umatrix_out'].shape[1], 4)
-    self.assertEqual(output['umatrix_out'].dtype, np.long)
+    self.assertEqual(output['umatrix_out'].dtype, np.dtype('intp'))
     for i in [0, 1, 3]:
       for j in range(100):
         self.assertEqual(x[j, i], output['umatrix_out'][j, i])
@@ -509,7 +509,7 @@ class TestPythonBinding(unittest.TestCase):
 
     self.assertEqual(output['umatrix_out'].shape[0], 3)
     self.assertEqual(output['umatrix_out'].shape[1], 4)
-    self.assertEqual(output['umatrix_out'].dtype, np.long)
+    self.assertEqual(output['umatrix_out'].dtype, np.dtype('intp'))
     self.assertEqual(output['umatrix_out'][0, 0], 1)
     self.assertEqual(output['umatrix_out'][0, 1], 2)
     self.assertEqual(output['umatrix_out'][0, 2], 6)
@@ -543,7 +543,7 @@ class TestPythonBinding(unittest.TestCase):
     self.assertEqual(output['umatrix_out'].shape[1], 4)
     self.assertEqual(len(x), 3)
     self.assertEqual(len(x[0]), 5)
-    self.assertEqual(output['umatrix_out'].dtype, np.long)
+    self.assertEqual(output['umatrix_out'].dtype, np.dtype('intp'))
     self.assertEqual(output['umatrix_out'][0, 0], 1)
     self.assertEqual(output['umatrix_out'][0, 1], 2)
     self.assertEqual(output['umatrix_out'][0, 2], 6)
@@ -612,7 +612,7 @@ class TestPythonBinding(unittest.TestCase):
                                  ucol_in=z)
 
     self.assertEqual(output['ucol_out'].shape[0], 100)
-    self.assertEqual(output['ucol_out'].dtype, np.long)
+    self.assertEqual(output['ucol_out'].dtype, np.dtype('intp'))
     for i in range(100):
       self.assertEqual(output['ucol_out'][i], x[i] * 2)
 
@@ -631,7 +631,7 @@ class TestPythonBinding(unittest.TestCase):
                                  copy_all_inputs=True)
 
     self.assertEqual(output['ucol_out'].shape[0], 100)
-    self.assertEqual(output['ucol_out'].dtype, np.long)
+    self.assertEqual(output['ucol_out'].dtype, np.dtype('intp'))
     for i in range(100):
       self.assertEqual(output['ucol_out'][i], x[i] * 2)
 
@@ -690,7 +690,7 @@ class TestPythonBinding(unittest.TestCase):
                                  urow_in=z)
 
     self.assertEqual(output['urow_out'].shape[0], 100)
-    self.assertEqual(output['urow_out'].dtype, np.long)
+    self.assertEqual(output['urow_out'].dtype, np.dtype('intp'))
 
     for i in range(100):
       self.assertEqual(output['urow_out'][i], x[i] * 2)
@@ -710,7 +710,7 @@ class TestPythonBinding(unittest.TestCase):
                                  copy_all_inputs=True)
 
     self.assertEqual(output['urow_out'].shape[0], 100)
-    self.assertEqual(output['urow_out'].dtype, np.long)
+    self.assertEqual(output['urow_out'].dtype, np.dtype('intp'))
 
     for i in range(100):
       self.assertEqual(output['urow_out'][i], x[i] * 2)
@@ -923,7 +923,7 @@ class TestPythonBinding(unittest.TestCase):
                                  s_umatrix_in=z)
 
     self.assertEqual(output['s_umatrix_out'].shape[0], 100)
-    self.assertEqual(output['s_umatrix_out'].dtype, np.long)
+    self.assertEqual(output['s_umatrix_out'].dtype, np.dtype('intp'))
 
     for i in range(100):
       self.assertEqual(output['s_umatrix_out'][i, 0], x[i] * 2)
@@ -943,7 +943,7 @@ class TestPythonBinding(unittest.TestCase):
                                  copy_all_inputs=True)
 
     self.assertEqual(output['s_umatrix_out'].shape[0], 100)
-    self.assertEqual(output['s_umatrix_out'].dtype, np.long)
+    self.assertEqual(output['s_umatrix_out'].dtype, np.dtype('intp'))
 
     for i in range(100):
       self.assertEqual(output['s_umatrix_out'][i, 0], x[i] * 2)
@@ -1003,7 +1003,7 @@ class TestPythonBinding(unittest.TestCase):
                                  ucol_in=z)
 
     self.assertEqual(output['ucol_out'].shape[0], 100)
-    self.assertEqual(output['ucol_out'].dtype, np.long)
+    self.assertEqual(output['ucol_out'].dtype, np.dtype('intp'))
     for i in range(100):
       self.assertEqual(output['ucol_out'][i], x[i] * 2)
 
@@ -1022,7 +1022,7 @@ class TestPythonBinding(unittest.TestCase):
                                  copy_all_inputs=True)
 
     self.assertEqual(output['ucol_out'].shape[0], 100)
-    self.assertEqual(output['ucol_out'].dtype, np.long)
+    self.assertEqual(output['ucol_out'].dtype, np.dtype('intp'))
     for i in range(100):
       self.assertEqual(output['ucol_out'][i], x[i] * 2)
 
@@ -1081,7 +1081,7 @@ class TestPythonBinding(unittest.TestCase):
                                  urow_in=z)
 
     self.assertEqual(output['urow_out'].shape[0], 100)
-    self.assertEqual(output['urow_out'].dtype, np.long)
+    self.assertEqual(output['urow_out'].dtype, np.dtype('intp'))
 
     for i in range(100):
       self.assertEqual(output['urow_out'][i], x[i] * 2)
@@ -1101,7 +1101,7 @@ class TestPythonBinding(unittest.TestCase):
                                  copy_all_inputs=True)
 
     self.assertEqual(output['urow_out'].shape[0], 101)
-    self.assertEqual(output['urow_out'].dtype, np.long)
+    self.assertEqual(output['urow_out'].dtype, np.dtype('intp'))
 
     for i in range(101):
       self.assertEqual(output['urow_out'][i], x[0][i] * 2)
