@@ -5,9 +5,6 @@
  * Include Armadillo extensions which currently are not part of the main
  * Armadillo codebase.
  *
- * This will allow the use of the ccov() function (which performs the same
- * function as cov(trans(X)) but without the cost of computing trans(X)).  This
- * also gives sparse matrix support, if it is necessary.
  */
 #ifndef MLPACK_CORE_ARMA_EXTEND_ARMA_EXTEND_HPP
 #define MLPACK_CORE_ARMA_EXTEND_ARMA_EXTEND_HPP
@@ -39,11 +36,6 @@
     #endif
 #endif
 
-// Make sure that U64 and S64 support is enabled.
-#ifndef ARMA_USE_U64S64
-  #define ARMA_USE_U64S64
-#endif
-
 // Include everything we'll need for serialize().
 #include <boost/serialization/serialization.hpp>
 #include <boost/serialization/nvp.hpp>
@@ -51,19 +43,4 @@
 
 #include <armadillo>
 
-namespace arma {
-  // u64/s64
-  #include "hdf5_misc.hpp"
-
-  // ccov()
-  #include "op_ccov_proto.hpp"
-  #include "op_ccov_meat.hpp"
-  #include "glue_ccov_proto.hpp"
-  #include "glue_ccov_meat.hpp"
-  #include "fn_ccov.hpp"
-
-  // inplace_reshape()
-  #include "fn_inplace_reshape.hpp"
-};
-
 #endif
