@@ -63,6 +63,13 @@ inline void FixedRandomSeed()
   srand((unsigned int) seed);
   arma::arma_rng::set_seed(seed);
 }
+
+inline void CustomRandomSeed(const size_t seed)
+{
+  randGen.seed((uint32_t) seed);
+  srand((unsigned int) seed);
+  arma::arma_rng::set_seed(seed);
+}
 #endif
 
 /**
@@ -81,6 +88,17 @@ inline double Random(const double lo, const double hi)
   return lo + (hi - lo) * randUniformDist(randGen);
 }
 
+/**
+ * Generates a 0/1 specified by the input.
+ */
+inline double RandBernoulli(const double input)
+{
+  if (Random() < input)
+    return 1;
+  else
+    return 0;
+}
+
 /**
  * Generates a uniform random integer.
  */
