@@ -23,17 +23,33 @@ using namespace std;
  * Construct a ProgramDoc object.  When constructed, it will register itself
  * with CLI.  A fatal error will be thrown if more than one is constructed.
  *
- * @param programName Short string representing the name of the program.
- * @param documentation Long string containing documentation on how to use the
- *    program and what it is.  No newline characters are necessary; this is
- *    taken care of by CLI later.
  * @param defaultModule Name of the default module.
+ * @param shortDocumentation A short two-sentence description of the program,
+ *     what it does, and what it is useful for.
+ * @param documentation Long string containing documentation on how to use the
+ *     program and what it is.  No newline characters are necessary; this is
+ *     taken care of by CLI later.
+ * @param seeAlso A set of pairs of strings with useful "see also"
+ *     information; each pair is <description, url>.
  */
-ProgramDoc::ProgramDoc(const std::string& programName,
-                       const std::function<std::string()>& documentation) :
+ProgramDoc::ProgramDoc(
+    const std::string& programName,
+    const std::string& shortDocumentation,
+    const std::function<std::string()>& documentation,
+    const std::vector<std::pair<std::string, std::string>>& seeAlso) :
     programName(programName),
-    documentation(documentation)
+    shortDocumentation(shortDocumentation),
+    documentation(documentation),
+    seeAlso(seeAlso)
 {
   // Register this with CLI.
   CLI::RegisterProgramDoc(this);
 }
+
+/**
+ * Construct an empty ProgramDoc object.
+ */
+ProgramDoc::ProgramDoc()
+{
+  CLI::RegisterProgramDoc(this);
+}
