@@ -77,10 +77,17 @@ PROGRAM_INFO("AdaBoost",
     "predictions for a given test dataset.  A test dataset may be specified "
     "with the " + PRINT_PARAM_STRING("test") + " parameter.  The predicted "
     "classes for each point in the test dataset are output to the " +
-    PRINT_PARAM_STRING("output") + " output parameter.  The AdaBoost model "
-    "itself is output to the " + PRINT_PARAM_STRING("output_model") +
+    PRINT_PARAM_STRING("predictions") + " output parameter.  The AdaBoost "
+    "model itself is output to the " + PRINT_PARAM_STRING("output_model") +
     " output parameter."
     "\n\n"
+    "Note: the following parameter is deprecated and "
+    "will be removed in mlpack 4.0.0: " + PRINT_PARAM_STRING("output") +
+    "."
+    "\n"
+    "Use " + PRINT_PARAM_STRING("predictions") + " instead of " +
+    PRINT_PARAM_STRING("output") + '.' +
+    "\n\n"
     "For example, to run AdaBoost on an input dataset " +
     PRINT_DATASET("data") + " with perceptrons as the weak learner type, "
     "storing the trained model in " + PRINT_MODEL("model") + ", one could "
@@ -95,7 +102,7 @@ PROGRAM_INFO("AdaBoost",
     PRINT_DATASET("predictions") + " with the following command: "
     "\n\n" +
     PRINT_CALL("adaboost", "input_model", "model", "test", "test_data",
-        "output", "predictions"),
+        "predictions", "predictions"),
     // See also...
     SEE_ALSO("AdaBoost on Wikipedia", "https://en.wikipedia.org/wiki/AdaBoost"),
     SEE_ALSO("Improved boosting algorithms using confidence-rated predictions "
@@ -111,7 +118,9 @@ PARAM_UROW_IN("labels", "Labels for the training set.", "l");
 
 // Classification options.
 PARAM_MATRIX_IN("test", "Test dataset.", "T");
+// PARAM_UROW_OUT("output") is deprecated and will be removed in mlpack 4.0.0.
 PARAM_UROW_OUT("output", "Predicted labels for the test set.", "o");
+PARAM_UROW_OUT("predictions", "Predicted labels for the test set.", "P");
 
 // Training options.
 PARAM_INT_IN("iterations", "The maximum number of boosting iterations to be run"
@@ -155,10 +164,11 @@ static void mlpackMain()
   if (CLI::HasParam("input_model"))
     RequireAtLeastOnePassed({ "test" }, false, "no task will be performed");
 
-  RequireAtLeastOnePassed({ "output_model", "output" }, false,
+  RequireAtLeastOnePassed({ "output_model", "output", "predictions" }, false,
       "no results will be saved");
 
-  ReportIgnoredParam({{ "test", false }}, "output");
+  // "output" will be removed in mlpack 4.0.0.
+  ReportIgnoredParam({{ "test", false }}, "predictions");
 
   AdaBoostModel* m;
   if (CLI::HasParam("training"))
@@ -230,7 +240,11 @@ static void mlpackMain()
     Row<size_t> results;
     data::RevertLabels(predictedLabels, m->Mappings(), results);
 
-    CLI::GetParam<arma::Row<size_t>>("output") = std::move(results);
+    // Save the predicted labels.
+    if (CLI::HasParam("output"))
+      CLI::GetParam<arma::Row<size_t>>("output") = results;
+    if (CLI::HasParam("predictions"))
+      CLI::GetParam<arma::Row<size_t>>("predictions") = std::move(results);
   }
 
   CLI::GetParam<AdaBoostModel*>("output_model") = m;
