@@ -1,6 +1,7 @@
 /**
  * @file concat_impl.hpp
  * @author Marcus Edel
+ * @author Mehul Kumar Nirala
  *
  * Implementation of the Concat class, which acts as a concatenation contain.
  *
@@ -25,11 +26,66 @@ namespace ann /** Artificial Neural Network. */ {
 template<typename InputDataType, typename OutputDataType,
          typename... CustomLayers>
 Concat<InputDataType, OutputDataType, CustomLayers...>::Concat(
-    const bool model, const bool run) : model(model), run(run)
+    const bool model, const bool run) :
+    axis(0),
+    useAxis(false),
+    model(model),
+    run(run),
+    channels(1)
 {
   parameters.set_size(0, 0);
 }
 
+template<typename InputDataType, typename OutputDataType,
+         typename... CustomLayers>
+Concat<InputDataType, OutputDataType, CustomLayers...>::Concat(
+    arma::Row<size_t>& inputSize,
+    const size_t axis,
+    const bool model,
+    const bool run) :
+    inputSize(inputSize),
+    axis(axis),
+    useAxis(true),
+    model(model),
+    run(run)
+{
+  parameters.set_size(0, 0);
+
+  // Parameters to help calculate the number of channels.
+  size_t oldColSize = 1, newColSize = 1;
+  // Axis is specified and useAxis is true.
+  if (useAxis)
+  {
+    // Axis is specified without input dimension.
+    // Throw an error.
+    if (inputSize.n_elem > 0)
+    {
+      // Calculate rowSize, newColSize based on the axis
+      // of concatenation. Finally concat along cols and
+      // reshape to original format i.e. (input, batch_size).
+      size_t i = std::min(axis + 1, (size_t) inputSize.n_elem);
+      for (; i < inputSize.n_elem; ++i)
+      {
+        newColSize *= inputSize[i];
+      }
+    }
+    else
+    {
+      throw std::logic_error("Input dimensions not specified.");
+    }
+  }
+  else
+  {
+    channels = 1;
+  }
+  if (newColSize <= 0)
+  {
+      throw std::logic_error("Col size is zero.");
+  }
+  channels = newColSize / oldColSize;
+  inputSize.clear();
+}
+
 template<typename InputDataType, typename OutputDataType,
          typename... CustomLayers>
 Concat<InputDataType, OutputDataType, CustomLayers...>::~Concat()
@@ -54,13 +110,24 @@ void Concat<InputDataType, OutputDataType, CustomLayers...>::Forward(
           network[i]);
     }
   }
-  // Vertically concatentate output from each layer.
+
   output = boost::apply_visitor(outputParameterVisitor, network.front());
+
+  // Reshape output to incorporate the channels.
+  output.reshape(output.n_rows / channels, output.n_cols * channels);
+
   for (size_t i = 1; i < network.size(); ++i)
   {
-    output = arma::join_cols(output,
-        boost::apply_visitor(outputParameterVisitor, network[i]));
+    arma::Mat<eT> out = boost::apply_visitor(outputParameterVisitor,
+        network[i]);
+
+    out.reshape(out.n_rows / channels, out.n_cols * channels);
+
+    // Vertically concatentate output from each layer.
+    output = arma::join_cols(output, out);
   }
+  // Reshape output to its original shape.
+  output.reshape(output.n_rows * channels, output.n_cols / channels);
 }
 
 template<typename InputDataType, typename OutputDataType,
@@ -72,13 +139,18 @@ void Concat<InputDataType, OutputDataType, CustomLayers...>::Backward(
   size_t rowCount = 0;
   if (run)
   {
-    arma::mat delta;
+    arma::Mat<eT> delta;
+    gy.reshape(gy.n_rows / channels, gy.n_cols * channels);
     for (size_t i = 0; i < network.size(); ++i)
     {
       // Use rows from the error corresponding to the output from each layer.
       size_t rows = boost::apply_visitor(
           outputParameterVisitor, network[i]).n_rows;
-      delta = gy.rows(rowCount, rowCount + rows - 1);
+
+      // Extract from gy the parameters for the i-th network.
+      delta = gy.rows(rowCount / channels, (rowCount + rows) / channels - 1);
+      delta.reshape(delta.n_rows * channels, delta.n_cols / channels);
+
       boost::apply_visitor(BackwardVisitor(std::move(
           boost::apply_visitor(outputParameterVisitor,
           network[i])), std::move(delta), std::move(
@@ -91,6 +163,7 @@ void Concat<InputDataType, OutputDataType, CustomLayers...>::Backward(
     {
       g += boost::apply_visitor(deltaVisitor, network[i]);
     }
+    gy.reshape(gy.n_rows * channels, gy.n_cols / channels);
   }
   else
   {
@@ -106,16 +179,28 @@ void Concat<InputDataType, OutputDataType, CustomLayers...>::Backward(
     const size_t index)
 {
   size_t rowCount = 0, rows = 0;
+
   for (size_t i = 0; i < index; ++i)
   {
-    rowCount += boost::apply_visitor(outputParameterVisitor, network[i]).n_rows;
+    rowCount += boost::apply_visitor(
+        outputParameterVisitor, network[i]).n_rows;
   }
   rows = boost::apply_visitor(outputParameterVisitor, network[index]).n_rows;
-  arma::mat delta = gy.rows(rowCount, rowCount + rows - 1);
+
+  // Reshape gy to extract the i-th layer gy.
+  gy.reshape(gy.n_rows / channels, gy.n_cols * channels);
+
+  arma::Mat<eT> delta = gy.rows(rowCount / channels, (rowCount + rows) /
+      channels - 1);
+  delta.reshape(delta.n_rows * channels, delta.n_cols / channels);
+
   boost::apply_visitor(BackwardVisitor(std::move(boost::apply_visitor(
       outputParameterVisitor, network[index])), std::move(delta), std::move(
       boost::apply_visitor(deltaVisitor, network[index]))), network[index]);
 
+  // Reshape gy to its original shape.
+  gy.reshape(gy.n_rows * channels, gy.n_cols / channels);
+
   g = boost::apply_visitor(deltaVisitor, network[index]);
 }
 
@@ -130,14 +215,25 @@ void Concat<InputDataType, OutputDataType, CustomLayers...>::Gradient(
   if (run)
   {
     size_t rowCount = 0;
+    // Reshape error to extract the i-th layer error.
+    error.reshape(error.n_rows / channels, error.n_cols * channels);
     for (size_t i = 0; i < network.size(); ++i)
     {
       size_t rows = boost::apply_visitor(
           outputParameterVisitor, network[i]).n_rows;
+
+      // Extract from error the parameters for the i-th network.
+      arma::Mat<eT> err = error.rows(rowCount / channels, (rowCount + rows) /
+          channels - 1);
+      err.reshape(err.n_rows * channels, err.n_cols / channels);
+
       boost::apply_visitor(GradientVisitor(std::move(input),
-          std::move(error.rows(rowCount, rowCount + rows - 1))), network[i]);
+          std::move(err)), network[i]);
       rowCount += rows;
     }
+
+    // Reshape error to its original shape.
+    error.reshape(error.n_rows * channels, error.n_cols / channels);
   }
 }
 
@@ -153,12 +249,21 @@ void Concat<InputDataType, OutputDataType, CustomLayers...>::Gradient(
   size_t rowCount = 0;
   for (size_t i = 0; i < index; ++i)
   {
-    rowCount += boost::apply_visitor(outputParameterVisitor, network[i]).n_rows;
+    rowCount += boost::apply_visitor(outputParameterVisitor,
+        network[i]).n_rows;
   }
   size_t rows = boost::apply_visitor(
       outputParameterVisitor, network[index]).n_rows;
+
+  error.reshape(error.n_rows / channels, error.n_cols * channels);
+  arma::Mat<eT> err = error.rows(rowCount / channels, (rowCount + rows) /
+      channels - 1);
+  err.reshape(err.n_rows * channels, err.n_cols / channels);
+
   boost::apply_visitor(GradientVisitor(std::move(input),
-      std::move(error.rows(rowCount, rowCount + rows - 1))), network[index]);
+      std::move(err)), network[index]);
+
+  error.reshape(error.n_rows * channels, error.n_cols / channels);
 }
 
 template<typename InputDataType, typename OutputDataType,
