@@ -19,19 +19,25 @@
 #include "convolution.hpp"
 #include "dropconnect.hpp"
 #include "glimpse.hpp"
+#include "highway.hpp"
 #include "layer_norm.hpp"
 #include "layer_types.hpp"
 #include "linear.hpp"
 #include "linear_no_bias.hpp"
 #include "lstm.hpp"
+#include "minibatch_discrimination.hpp"
 #include "multiply_merge.hpp"
+#include "padding.hpp"
 #include "gru.hpp"
 #include "fast_lstm.hpp"
 #include "recurrent.hpp"
 #include "recurrent_attention.hpp"
+#include "reparametrization.hpp"
 #include "sequential.hpp"
+#include "subview.hpp"
 #include "concat.hpp"
 #include "vr_class_reward.hpp"
 #include "transposed_convolution.hpp"
+#include "weight_norm.hpp"
 
 #endif
