@@ -40,12 +40,16 @@
 #include <mlpack/methods/ann/layer/reparametrization.hpp>
 #include <mlpack/methods/ann/layer/select.hpp>
 #include <mlpack/methods/ann/layer/subview.hpp>
+#include <mlpack/methods/ann/layer/virtual_batch_norm.hpp>
 
 // Convolution modules.
 #include <mlpack/methods/ann/convolution_rules/border_modes.hpp>
 #include <mlpack/methods/ann/convolution_rules/naive_convolution.hpp>
 #include <mlpack/methods/ann/convolution_rules/fft_convolution.hpp>
 
+// Regularizers.
+#include <mlpack/methods/ann/regularizer/no_regularizer.hpp>
+
 // Loss function modules.
 #include <mlpack/methods/ann/loss_functions/negative_log_likelihood.hpp>
 
@@ -56,13 +60,32 @@ template<typename InputDataType, typename OutputDataType> class BatchNorm;
 template<typename InputDataType, typename OutputDataType> class DropConnect;
 template<typename InputDataType, typename OutputDataType> class Glimpse;
 template<typename InputDataType, typename OutputDataType> class LayerNorm;
-template<typename InputDataType, typename OutputDataType> class Linear;
-template<typename InputDataType, typename OutputDataType> class LinearNoBias;
 template<typename InputDataType, typename OutputDataType> class LSTM;
 template<typename InputDataType, typename OutputDataType> class GRU;
 template<typename InputDataType, typename OutputDataType> class FastLSTM;
 template<typename InputDataType, typename OutputDataType> class VRClassReward;
 template<typename InputDataType, typename OutputDataType> class Concatenate;
+template<typename InputDataType, typename OutputDataType> class Padding;
+
+template<typename InputDataType,
+         typename OutputDataType,
+         typename RegularizerType>
+class Linear;
+
+template<typename InputDataType,
+         typename OutputDataType,
+         typename RegularizerType>
+class LinearNoBias;
+
+template<typename InputDataType,
+         typename OutputDataType
+>
+class VirtualBatchNorm;
+
+template<typename InputDataType,
+         typename OutputDataType
+>
+class MiniBatchDiscrimination;
 
 template<typename InputDataType,
          typename OutputDataType
@@ -82,6 +105,12 @@ template<typename InputDataType,
 >
 class Sequential;
 
+template<typename InputDataType,
+         typename OutputDataType,
+         typename... CustomLayers
+>
+class Highway;
+
 template<typename InputDataType,
          typename OutputDataType,
          typename... CustomLayers
@@ -140,6 +169,25 @@ template<typename InputDataType,
 >
 class MultiplyMerge;
 
+template <typename InputDataType,
+          typename OutputDataType,
+          typename... CustomLayers
+>
+class WeightNorm;
+
+using MoreTypes = boost::variant<
+        Recurrent<arma::mat, arma::mat>*,
+        RecurrentAttention<arma::mat, arma::mat>*,
+        ReinforceNormal<arma::mat, arma::mat>*,
+        Reparametrization<arma::mat, arma::mat>*,
+        Select<arma::mat, arma::mat>*,
+        Sequential<arma::mat, arma::mat, false>*,
+        Sequential<arma::mat, arma::mat, true>*,
+        Subview<arma::mat, arma::mat>*,
+        VRClassReward<arma::mat, arma::mat>*,
+        VirtualBatchNorm<arma::mat, arma::mat>*
+>;
+
 template <typename... CustomLayers>
 using LayerTypes = boost::variant<
     Add<arma::mat, arma::mat>*,
@@ -164,7 +212,7 @@ using LayerTypes = boost::variant<
                 NaiveConvolution<FullConvolution>,
                 NaiveConvolution<ValidConvolution>, arma::mat, arma::mat>*,
     TransposedConvolution<NaiveConvolution<ValidConvolution>,
-            NaiveConvolution<FullConvolution>,
+            NaiveConvolution<ValidConvolution>,
             NaiveConvolution<ValidConvolution>, arma::mat, arma::mat>*,
     DropConnect<arma::mat, arma::mat>*,
     Dropout<arma::mat, arma::mat>*,
@@ -173,12 +221,13 @@ using LayerTypes = boost::variant<
     FlexibleReLU<arma::mat, arma::mat>*,
     Glimpse<arma::mat, arma::mat>*,
     HardTanH<arma::mat, arma::mat>*,
+    Highway<arma::mat, arma::mat>*,
     Join<arma::mat, arma::mat>*,
     LayerNorm<arma::mat, arma::mat>*,
     LeakyReLU<arma::mat, arma::mat>*,
     CReLU<arma::mat, arma::mat>*,
-    Linear<arma::mat, arma::mat>*,
-    LinearNoBias<arma::mat, arma::mat>*,
+    Linear<arma::mat, arma::mat, NoRegularizer>*,
+    LinearNoBias<arma::mat, arma::mat, NoRegularizer>*,
     LogSoftMax<arma::mat, arma::mat>*,
     Lookup<arma::mat, arma::mat>*,
     LSTM<arma::mat, arma::mat>*,
@@ -186,19 +235,14 @@ using LayerTypes = boost::variant<
     FastLSTM<arma::mat, arma::mat>*,
     MaxPooling<arma::mat, arma::mat>*,
     MeanPooling<arma::mat, arma::mat>*,
+    MiniBatchDiscrimination<arma::mat, arma::mat>*,
     MultiplyConstant<arma::mat, arma::mat>*,
     MultiplyMerge<arma::mat, arma::mat>*,
     NegativeLogLikelihood<arma::mat, arma::mat>*,
+    Padding<arma::mat, arma::mat>*,
     PReLU<arma::mat, arma::mat>*,
-    Recurrent<arma::mat, arma::mat>*,
-    RecurrentAttention<arma::mat, arma::mat>*,
-    ReinforceNormal<arma::mat, arma::mat>*,
-    Reparametrization<arma::mat, arma::mat>*,
-    Select<arma::mat, arma::mat>*,
-    Sequential<arma::mat, arma::mat, false>*,
-    Sequential<arma::mat, arma::mat, true>*,
-    Subview<arma::mat, arma::mat>*,
-    VRClassReward<arma::mat, arma::mat>*,
+    WeightNorm<arma::mat, arma::mat>*,
+    MoreTypes,
     CustomLayers*...
 >;
 
