@@ -19,12 +19,7 @@ namespace mlpack {
 namespace ann /** Artificial Neural Network. */ {
 
 /**
- * An implementation of a lstm network layer.
- *
- * This class allows specification of the type of the activation functions used
- * for the gates and cells and also of the type of the function used to
- * initialize and update the peephole weights.
-
+ * Implementation of the LSTM module class.
  * The implementation corresponds to the following algorithm:
  *
  * @f{eqnarray}{
@@ -36,6 +31,10 @@ namespace ann /** Artificial Neural Network. */ {
  * h &=& o \cdot tanh(c)
  * @f}
  *
+ * Note that if an LSTM layer is desired as the first layer of a neural network,
+ * an IdentityLayer should be added to the network as the first layer, and then
+ * the LSTM layer should be added.
+ *
  * For more information, see the following.
  *
  * @code
@@ -78,7 +77,7 @@ class LSTM
        const size_t rho = std::numeric_limits<size_t>::max());
 
   /**
-   * Ordinary feed forward pass of a neural network, evaluating the function
+   * Ordinary feed-forward pass of a neural network, evaluating the function
    * f(x) by propagating the activity forward through f.
    *
    * @param input Input data used for evaluating the specified function.
@@ -87,6 +86,21 @@ class LSTM
   template<typename InputType, typename OutputType>
   void Forward(InputType&& input, OutputType&& output);
 
+  /**
+   * Ordinary feed-forward pass of a neural network, evaluating the function
+   * f(x) by propagating the activity forward through f.
+   *
+   * @param input Input data used for evaluating the specified function.
+   * @param output Resulting output activation.
+   * @param cellState Cell state of the LSTM.
+   * @param useCellState Use the cellState passed in the LSTM cell.
+   */
+  template<typename InputType, typename OutputType>
+  void Forward(InputType&& input,
+               OutputType&& output,
+               OutputType&& cellState,
+               bool useCellState = false);
+
   /**
    * Ordinary feed backward pass of a neural network, calculating the function
    * f(x) by propagating x backwards trough f. Using the results from the feed
@@ -136,11 +150,6 @@ class LSTM
   //! Modify the parameters.
   OutputDataType& Parameters() { return weights; }
 
-  //! Get the input parameter.
-  InputDataType const& InputParameter() const { return inputParameter; }
-  //! Modify the input parameter.
-  InputDataType& InputParameter() { return inputParameter; }
-
   //! Get the output parameter.
   OutputDataType const& OutputParameter() const { return outputParameter; }
   //! Modify the output parameter.
@@ -206,9 +215,6 @@ class LSTM
   //! Locally-stored gradient object.
   OutputDataType grad;
 
-  //! Locally-stored input parameter object.
-  InputDataType inputParameter;
-
   //! Locally-stored output parameter object.
   OutputDataType outputParameter;
 
