@@ -159,10 +159,24 @@ void LSTM<InputDataType, OutputDataType>::Reset()
       offset, outSize, 1, false, false);
 }
 
+// Forward when cellState is not needed.
 template<typename InputDataType, typename OutputDataType>
 template<typename InputType, typename OutputType>
 void LSTM<InputDataType, OutputDataType>::Forward(
     InputType&& input, OutputType&& output)
+{
+  //! Locally-stored cellState.
+  OutputType cellState;
+  Forward(std::move(input), std::move(output), std::move(cellState), false);
+}
+
+// Forward when cellState is needed overloaded LSTM::Forward().
+template<typename InputDataType, typename OutputDataType>
+template<typename InputType, typename OutputType>
+void LSTM<InputDataType, OutputDataType>::Forward(InputType&& input,
+                                                  OutputType&& output,
+                                                  OutputType&& cellState,
+                                                  bool useCellState)
 {
   // Check if the batch size changed, the number of cols is defines the input
   // batch size.
@@ -187,6 +201,18 @@ void LSTM<InputDataType, OutputDataType>::Forward(
 
   if (forwardStep > 0)
   {
+    if (useCellState)
+    {
+      if (!cellState.is_empty())
+      {
+        cell.cols(forwardStep - batchSize,
+            forwardStep - batchSize + batchStep) = cellState;
+      }
+      else
+      {
+        throw std::runtime_error("Cell parameter is empty.");
+      }
+    }
     inputGate.cols(forwardStep, forwardStep + batchStep) +=
         arma::repmat(cell2GateInputWeight, 1, batchSize) %
         cell.cols(forwardStep - batchSize, forwardStep - batchSize + batchStep);
@@ -249,6 +275,9 @@ void LSTM<InputDataType, OutputDataType>::Forward(
   output = OutputType(outParameter.memptr() +
       (forwardStep + batchSize) * outSize, outSize, batchSize, false, false);
 
+  cellState = OutputType(cell.memptr() +
+      forwardStep * outSize, outSize, batchSize, false, false);
+
   forwardStep += batchSize;
   if ((forwardStep / batchSize) == bpttSteps)
   {
