@@ -74,11 +74,6 @@ class MeanPooling
                 arma::Mat<eT>&& gy,
                 arma::Mat<eT>&& g);
 
-  //! Get the input parameter.
-  InputDataType const& InputParameter() const { return inputParameter; }
-  //! Modify the input parameter.
-  InputDataType& InputParameter() { return inputParameter; }
-
   //! Get the output parameter.
   OutputDataType const& OutputParameter() const { return outputParameter; }
   //! Modify the output parameter.
@@ -177,12 +172,6 @@ class MeanPooling
     }
   }
 
-  //! Locally-stored number of input units.
-  size_t inSize;
-
-  //! Locally-stored number of output units.
-  size_t outSize;
-
   //! Locally-stored width of the pooling window.
   size_t kW;
 
@@ -195,6 +184,15 @@ class MeanPooling
   //! Locally-stored height of the stride operation.
   size_t dH;
 
+  //! Rounding operation used.
+  bool floor;
+
+  //! Locally-stored number of input channels.
+  size_t inSize;
+
+  //! Locally-stored number of output channels.
+  size_t outSize;
+
   //! Locally-stored input width.
   size_t inputWidth;
 
@@ -210,15 +208,15 @@ class MeanPooling
   //! Locally-stored reset parameter used to initialize the module once.
   bool reset;
 
-  //! Rounding operation used.
-  bool floor;
-
   //! If true use maximum a posteriori during the forward pass.
   bool deterministic;
 
   //! Locally-stored stored rounding offset.
   size_t offset;
 
+  //! Locally-stored number of input units.
+  size_t batchSize;
+
   //! Locally-stored output parameter.
   arma::cube outputTemp;
 
@@ -234,9 +232,6 @@ class MeanPooling
   //! Locally-stored gradient object.
   OutputDataType gradient;
 
-  //! Locally-stored input parameter object.
-  InputDataType inputParameter;
-
   //! Locally-stored output parameter object.
   OutputDataType outputParameter;
 }; // class MeanPooling
