@@ -52,6 +52,7 @@ PROGRAM_INFO("Gaussian Mixture Model (GMM) Training",
     " specified with the " + PRINT_PARAM_STRING("input_model") + " parameter."
     " Otherwise, the model is initialized by running k-means on the data.  The "
     "k-means clustering initialization can be controlled with the " +
+    PRINT_PARAM_STRING("kmeans_max_iterations") + ", " +
     PRINT_PARAM_STRING("refined_start") + ", " +
     PRINT_PARAM_STRING("samplings") + ", and " +
     PRINT_PARAM_STRING("percentage") + " parameters.  If " +
@@ -120,6 +121,8 @@ PARAM_DOUBLE_IN("noise", "Variance of zero-mean Gaussian noise to add to data.",
     "N", 0);
 
 // Parameters for k-means initialization.
+PARAM_INT_IN("kmeans_max_iterations", "Maximum number of iterations for the "
+    "k-means algorithm (used to initialize EM).", "k", 1000);
 PARAM_FLAG("refined_start", "During the initialization, use refined initial "
     "positions for k-means clustering (Bradley and Fayyad, 1998).", "r");
 PARAM_INT_IN("samplings", "If using --refined_start, specify the number of "
@@ -145,12 +148,20 @@ static void mlpackMain()
       "number of Gaussians must be positive");
   const int gaussians = CLI::GetParam<int>("gaussians");
 
+  RequireParamValue<int>("trials", [](int x) { return x > 0; }, true,
+      "trials must be greater than 0");
+
   ReportIgnoredParam({{ "diagonal_covariance", true }}, "no_force_positive");
   RequireAtLeastOnePassed({ "output_model" }, false, "no model will be saved");
 
   RequireParamValue<double>("noise", [](double x) { return x >= 0.0; }, true,
       "variance of noise must be greater than or equal to 0");
 
+  RequireParamValue<int>("max_iterations", [](int x) { return x >= 0; }, true,
+      "max_iterations must be greater than or equal to 0");
+  RequireParamValue<int>("kmeans_max_iterations", [](int x) { return x >= 0; },
+      true, "kmeans_max_iterations must be greater than or equal to 0");
+
   arma::mat dataPoints = std::move(CLI::GetParam<arma::mat>("input"));
 
   // Do we need to add noise to the dataset?
@@ -187,6 +198,8 @@ static void mlpackMain()
   const double tolerance = CLI::GetParam<double>("tolerance");
   const bool forcePositive = !CLI::HasParam("no_force_positive");
   const bool diagonalCovariance = CLI::HasParam("diagonal_covariance");
+  const size_t kmeansMaxIterations =
+      (size_t) CLI::GetParam<int>("kmeans_max_iterations");
 
   // This gets a bit weird because we need different types depending on whether
   // --refined_start is specified.
@@ -204,8 +217,7 @@ static void mlpackMain()
 
     typedef KMeans<metric::SquaredEuclideanDistance, RefinedStart> KMeansType;
 
-    // These are default parameters.
-    KMeansType k(1000, metric::SquaredEuclideanDistance(),
+    KMeansType k(kmeansMaxIterations, metric::SquaredEuclideanDistance(),
         RefinedStart(samplings, percentage));
 
     // Depending on the value of forcePositive and diagonalCovariance, we have
@@ -278,9 +290,9 @@ static void mlpackMain()
 
       // Compute the parameters of the model using the EM algorithm.
       Timer::Start("em");
-      EMFit<kmeans::KMeans<>, PositiveDefiniteConstraint,
+      EMFit<KMeans<>, PositiveDefiniteConstraint,
           distribution::DiagonalGaussianDistribution> em(maxIterations,
-          tolerance);
+          tolerance, KMeans<>(kmeansMaxIterations));
 
       likelihood = dgmm.Train(dataPoints, CLI::GetParam<int>("trials"), false,
           em);
@@ -299,7 +311,7 @@ static void mlpackMain()
     {
       // Compute the parameters of the model using the EM algorithm.
       Timer::Start("em");
-      EMFit<> em(maxIterations, tolerance);
+      EMFit<> em(maxIterations, tolerance, KMeans<>(kmeansMaxIterations));
       likelihood = gmm->Train(dataPoints, CLI::GetParam<int>("trials"), false,
           em);
       Timer::Stop("em");
@@ -308,7 +320,8 @@ static void mlpackMain()
     {
       // Compute the parameters of the model using the EM algorithm.
       Timer::Start("em");
-      EMFit<KMeans<>, NoConstraint> em(maxIterations, tolerance);
+      KMeans<> k(kmeansMaxIterations);
+      EMFit<KMeans<>, NoConstraint> em(maxIterations, tolerance, k);
       likelihood = gmm->Train(dataPoints, CLI::GetParam<int>("trials"), false,
           em);
       Timer::Stop("em");
