@@ -115,7 +115,7 @@ SingleTreeTraversalType>::NeighborSearch(const NeighborSearchMode mode,
                                          const double epsilon,
                                          const MetricType metric) :
     referenceTree(NULL),
-    referenceSet(new MatType()), // Empty matrix.
+    referenceSet(mode == NAIVE_MODE ? new MatType() : NULL), // Empty matrix.
     searchMode(mode),
     epsilon(epsilon),
     metric(metric),
@@ -129,7 +129,8 @@ SingleTreeTraversalType>::NeighborSearch(const NeighborSearchMode mode,
   // Build the tree on the empty dataset, if necessary.
   if (mode != NAIVE_MODE)
   {
-    referenceTree = BuildTree<Tree>(*referenceSet, oldFromNewReferences);
+    referenceTree = BuildTree<Tree>(std::move(arma::mat()),
+        oldFromNewReferences);
     referenceSet = &referenceTree->Dataset();
   }
 }
@@ -181,8 +182,7 @@ SingleTreeTraversalType>::NeighborSearch(NeighborSearch&& other) :
     treeNeedsReset(other.treeNeedsReset)
 {
   // Clear the other model.
-  other.referenceSet = new MatType();
-  other.referenceTree = BuildTree<Tree>(*other.referenceSet,
+  other.referenceTree = BuildTree<Tree>(std::move(MatType()),
       other.oldFromNewReferences);
   other.referenceSet = &other.referenceTree->Dataset();
   other.searchMode = DUAL_TREE_MODE,
@@ -219,15 +219,9 @@ NeighborSearch<SortPolicy,
 
   // Clean memory first.
   if (referenceTree)
-  {
     delete referenceTree;
-    referenceTree = NULL;
-  }
   else
-  {
     delete referenceSet;
-    referenceSet = NULL;
-  }
 
   oldFromNewReferences = other.oldFromNewReferences;
   referenceTree = other.referenceTree ? new Tree(*other.referenceTree) : NULL;
@@ -268,15 +262,9 @@ NeighborSearch<SortPolicy,
 
   // Clean memory first.
   if (referenceTree)
-  {
     delete referenceTree;
-    referenceTree = NULL;
-  }
   else
-  {
     delete referenceSet;
-    referenceSet = NULL;
-  }
 
   oldFromNewReferences = std::move(other.oldFromNewReferences);
   referenceTree = other.referenceTree;
@@ -288,8 +276,11 @@ NeighborSearch<SortPolicy,
   scores = other.scores;
   treeNeedsReset = other.treeNeedsReset;
 
-  // Reset the other object.
-  other.referenceTree = BuildTree<Tree>(*other.referenceSet,
+  // Reset the other object.  Clean memory if needed.
+  if (!other.referenceTree)
+    delete other.referenceSet;
+
+  other.referenceTree = BuildTree<Tree>(std::move(arma::mat()),
       other.oldFromNewReferences);
   other.referenceSet = &other.referenceTree->Dataset();
   other.searchMode = DUAL_TREE_MODE,
@@ -312,15 +303,9 @@ NeighborSearch<SortPolicy, MetricType, MatType, TreeType, DualTreeTraversalType,
 SingleTreeTraversalType>::~NeighborSearch()
 {
   if (referenceTree)
-  {
     delete referenceTree;
-    referenceTree = NULL;
-  }
   else
-  {
     delete referenceSet;
-    referenceSet = NULL;
-  }
 }
 
 template<typename SortPolicy,
@@ -344,7 +329,6 @@ DualTreeTraversalType, SingleTreeTraversalType>::Train(MatType referenceSetIn)
   else
   {
     delete referenceSet;
-    referenceSet = NULL;
   }
 
   // We may need to rebuild the tree.
@@ -379,12 +363,10 @@ DualTreeTraversalType, SingleTreeTraversalType>::Train(Tree referenceTree)
   {
     oldFromNewReferences.clear();
     delete this->referenceTree;
-    this->referenceTree = NULL;
   }
   else
   {
     delete this->referenceSet;
-    this->referenceSet = NULL;
   }
 
   this->referenceTree = new Tree(std::move(referenceTree));
@@ -979,7 +961,6 @@ DualTreeTraversalType, SingleTreeTraversalType>::serialize(
     if (Archive::is_loading::value && referenceSet)
     {
       delete referenceSet;
-      referenceSet = NULL;
     }
 
     ar & BOOST_SERIALIZATION_NVP(referenceSet);
@@ -1001,7 +982,6 @@ DualTreeTraversalType, SingleTreeTraversalType>::serialize(
     if (Archive::is_loading::value && referenceTree)
     {
       delete referenceTree;
-      referenceTree = NULL;
     }
 
     ar & BOOST_SERIALIZATION_NVP(referenceTree);
