@@ -2,6 +2,7 @@
  * @file distribution_test.cpp
  * @author Ryan Curtin
  * @author Yannis Mentekidis
+ * @author Rohan Raj
  *
  * Tests for the classes:
  *  * mlpack::distribution::DiscreteDistribution
@@ -15,11 +16,16 @@
  */
 #include <mlpack/core.hpp>
 
+#include <mlpack/core/dists/regression_distribution.hpp>
+#include <mlpack/core/metrics/mahalanobis_distance.hpp>
+
 #include <boost/test/unit_test.hpp>
 #include "test_tools.hpp"
+#include "serialization.hpp"
 
 using namespace mlpack;
 using namespace mlpack::distribution;
+using namespace mlpack::metric;
 using namespace mlpack::math;
 
 BOOST_AUTO_TEST_SUITE(DistributionTest);
@@ -188,6 +194,50 @@ BOOST_AUTO_TEST_CASE(MultiDiscreteDistributionTrainProTest)
   BOOST_REQUIRE_CLOSE(d.Probability("2 1 0"), 0.015625, 1e-5);
 }
 
+/**
+ * Test the LogProbability() function, for multiple points in the multivariate
+ * Discrete case.
+ */
+BOOST_AUTO_TEST_CASE(DiscreteLogProbabilityTest)
+{
+  // Same case as before.
+  DiscreteDistribution d("5 5");
+
+  arma::mat obs("0 2;"
+                "1 2;");
+
+  arma::vec logProb;
+
+  d.LogProbability(obs, logProb);
+
+  BOOST_REQUIRE_EQUAL(logProb.n_elem, 2);
+
+  BOOST_REQUIRE_CLOSE(logProb(0), -3.2188758248682, 1e-3);
+  BOOST_REQUIRE_CLOSE(logProb(1), -3.2188758248682, 1e-3);
+}
+
+/**
+ * Test the Probability() function, for multiple points in the multivariate
+ * Discrete case.
+ */
+BOOST_AUTO_TEST_CASE(DiscreteProbabilityTest)
+{
+  // Same case as before.
+  DiscreteDistribution d("5 5");
+
+  arma::mat obs("0 2;"
+                "1 2;");
+
+  arma::vec prob;
+
+  d.Probability(obs, prob);
+
+  BOOST_REQUIRE_EQUAL(prob.n_elem, 2);
+
+  BOOST_REQUIRE_CLOSE(prob(0), 0.0400000000000, 1e-3);
+  BOOST_REQUIRE_CLOSE(prob(1), 0.0400000000000, 1e-3);
+}
+
 /*********************************/
 /** Gaussian Distribution Tests **/
 /*********************************/
@@ -411,7 +461,7 @@ BOOST_AUTO_TEST_CASE(GaussianDistributionRandomTest)
 
   // Now make sure that reflects the actual distribution.
   arma::vec obsMean = arma::mean(obs, 1);
-  arma::mat obsCov = ccov(obs);
+  arma::mat obsCov = mlpack::math::ColumnCovariance(obs);
 
   // 10% tolerance because this can be noisy.
   BOOST_REQUIRE_CLOSE(obsMean[0], mean[0], 10.0);
@@ -446,7 +496,7 @@ BOOST_AUTO_TEST_CASE(GaussianDistributionTrainTest)
 
   // Find actual mean and covariance of data.
   arma::vec actualMean = arma::mean(observations, 1);
-  arma::mat actualCov = ccov(observations);
+  arma::mat actualCov = mlpack::math::ColumnCovariance(observations);
 
   d.Train(observations);
 
@@ -488,11 +538,11 @@ BOOST_AUTO_TEST_CASE(GaussianDistributionTrainWithProbabilitiesTest)
   GaussianDistribution guDist2;
   guDist2.Train(rdata);
 
-  BOOST_REQUIRE_CLOSE(guDist.Mean()[0], guDist2.Mean()[0], 5);
-  BOOST_REQUIRE_CLOSE(guDist.Covariance()[0], guDist2.Covariance()[0], 5);
+  BOOST_REQUIRE_CLOSE(guDist.Mean()[0], guDist2.Mean()[0], 6);
+  BOOST_REQUIRE_CLOSE(guDist.Covariance()[0], guDist2.Covariance()[0], 6);
 
-  BOOST_REQUIRE_CLOSE(guDist.Mean()[0], mean[0], 5);
-  BOOST_REQUIRE_CLOSE(guDist.Covariance()[0], cov[0], 5);
+  BOOST_REQUIRE_CLOSE(guDist.Mean()[0], mean[0], 6);
+  BOOST_REQUIRE_CLOSE(guDist.Covariance()[0], cov[0], 6);
 }
 
 /**
@@ -667,11 +717,11 @@ BOOST_AUTO_TEST_CASE(GammaDistributionTrainWithProbabilitiesTest)
   BOOST_REQUIRE_CLOSE(gDist2.Alpha(1), gDist.Alpha(1), 1.5);
   BOOST_REQUIRE_CLOSE(gDist2.Beta(1), gDist.Beta(1), 1.5);
 
-  BOOST_REQUIRE_CLOSE(alphaReal, gDist.Alpha(0), 2.5);
-  BOOST_REQUIRE_CLOSE(betaReal, gDist.Beta(0), 2.5);
+  BOOST_REQUIRE_CLOSE(alphaReal, gDist.Alpha(0), 3.0);
+  BOOST_REQUIRE_CLOSE(betaReal, gDist.Beta(0), 3.0);
 
-  BOOST_REQUIRE_CLOSE(alphaReal, gDist.Alpha(1), 2.5);
-  BOOST_REQUIRE_CLOSE(betaReal, gDist.Beta(1), 2.5);
+  BOOST_REQUIRE_CLOSE(alphaReal, gDist.Alpha(1), 3.0);
+  BOOST_REQUIRE_CLOSE(betaReal, gDist.Beta(1), 3.0);
 }
 
 /**
@@ -930,17 +980,21 @@ BOOST_AUTO_TEST_CASE(GammaDistributionLogProbabilityTest)
   // Train two 1-dimensional distributions.
   const arma::vec a1("2.0"), b1("0.9"), a2("3.1"), b2("1.4");
   arma::mat x1("2.0"), x2("2.94");
-  arma::vec prob1, prob2;
+  arma::vec logprob1, logprob2;
 
   // Evaluated at wolfram|alpha
   GammaDistribution d1(a1, b1);
-  d1.LogProbability(x1, prob1);
-  BOOST_REQUIRE_CLOSE(prob1(0), std::log(0.267575), 1e-3);
+  d1.LogProbability(x1, logprob1);
+  BOOST_REQUIRE_CLOSE(logprob1(0), std::log(0.267575), 1e-3);
 
   // Evaluated at wolfram|alpha
   GammaDistribution d2(a2, b2);
-  d2.LogProbability(x2, prob2);
-  BOOST_REQUIRE_CLOSE(prob2(0), std::log(0.189043), 1e-3);
+  d2.LogProbability(x2, logprob2);
+  BOOST_REQUIRE_CLOSE(logprob2(0), std::log(0.189043), 1e-3);
+
+  // Check that the overload that returns the log probability for
+  // 1 dimension agrees.
+  BOOST_REQUIRE_CLOSE(logprob2(0), d2.LogProbability(2.94, 0), 1e-5);
 
   // Combine into one 2-dimensional distribution.
   const arma::vec a3("2.0 3.1"), b3("0.9 1.4");
@@ -948,14 +1002,523 @@ BOOST_AUTO_TEST_CASE(GammaDistributionLogProbabilityTest)
   x3
     << 2.0 << 2.94 << arma::endr
     << 2.0 << 2.94;
-  arma::vec prob3;
+  arma::vec logprob3;
 
   // Expect that the 2-dimensional distribution returns the product of the
   // 1-dimensional distributions (evaluated at wolfram|alpha).
   GammaDistribution d3(a3, b3);
-  d3.LogProbability(x3, prob3);
-  BOOST_REQUIRE_CLOSE(prob3(0), std::log(0.04408), 1e-3);
-  BOOST_REQUIRE_CLOSE(prob3(1), std::log(0.026165), 1e-3);
+  d3.LogProbability(x3, logprob3);
+  BOOST_REQUIRE_CLOSE(logprob3(0), std::log(0.04408), 1e-3);
+  BOOST_REQUIRE_CLOSE(logprob3(1), std::log(0.026165), 1e-3);
+}
+
+/**
+ * Discrete Distribution serialization test.
+ */
+BOOST_AUTO_TEST_CASE(DiscreteDistributionTest)
+{
+  // I assume that I am properly saving vectors, so, this should be
+  // straightforward.
+  arma::vec prob;
+  prob.randu(12);
+  std::vector<arma::vec> probVector = std::vector<arma::vec>(1, prob);
+  DiscreteDistribution t(probVector);
+
+  DiscreteDistribution xmlT, textT, binaryT;
+
+  // Load and save with all serializers.
+  SerializeObjectAll(t, xmlT, textT, binaryT);
+
+  for (size_t i = 0; i < 12; ++i)
+  {
+    arma::vec obs(1);
+    obs[0] = i;
+    const double prob = t.Probability(obs);
+    if (prob == 0.0)
+    {
+      BOOST_REQUIRE_SMALL(xmlT.Probability(obs), 1e-8);
+      BOOST_REQUIRE_SMALL(textT.Probability(obs), 1e-8);
+      BOOST_REQUIRE_SMALL(binaryT.Probability(obs), 1e-8);
+    }
+    else
+    {
+      BOOST_REQUIRE_CLOSE(prob, xmlT.Probability(obs), 1e-8);
+      BOOST_REQUIRE_CLOSE(prob, textT.Probability(obs), 1e-8);
+      BOOST_REQUIRE_CLOSE(prob, binaryT.Probability(obs), 1e-8);
+    }
+  }
+}
+
+/**
+ * Gaussian Distribution serialization test.
+ */
+BOOST_AUTO_TEST_CASE(GaussianDistributionTest)
+{
+  arma::vec mean(10);
+  mean.randu();
+  // Generate a covariance matrix.
+  arma::mat cov;
+  cov.randu(10, 10);
+  cov = (cov * cov.t());
+
+  GaussianDistribution g(mean, cov);
+  GaussianDistribution xmlG, textG, binaryG;
+
+  SerializeObjectAll(g, xmlG, textG, binaryG);
+
+  BOOST_REQUIRE_EQUAL(g.Dimensionality(), xmlG.Dimensionality());
+  BOOST_REQUIRE_EQUAL(g.Dimensionality(), textG.Dimensionality());
+  BOOST_REQUIRE_EQUAL(g.Dimensionality(), binaryG.Dimensionality());
+
+  // First, check the means.
+  CheckMatrices(g.Mean(), xmlG.Mean(), textG.Mean(), binaryG.Mean());
+
+  // Now, check the covariance.
+  CheckMatrices(g.Covariance(), xmlG.Covariance(), textG.Covariance(),
+      binaryG.Covariance());
+
+  // Lastly, run some observations through and make sure the probability is the
+  // same.  This should test anything cached internally.
+  arma::mat randomObs;
+  randomObs.randu(10, 500);
+
+  for (size_t i = 0; i < 500; ++i)
+  {
+    const double prob = g.Probability(randomObs.unsafe_col(i));
+
+    if (prob == 0.0)
+    {
+      BOOST_REQUIRE_SMALL(xmlG.Probability(randomObs.unsafe_col(i)), 1e-8);
+      BOOST_REQUIRE_SMALL(textG.Probability(randomObs.unsafe_col(i)), 1e-8);
+      BOOST_REQUIRE_SMALL(binaryG.Probability(randomObs.unsafe_col(i)), 1e-8);
+    }
+    else
+    {
+      BOOST_REQUIRE_CLOSE(prob, xmlG.Probability(randomObs.unsafe_col(i)),
+          1e-8);
+      BOOST_REQUIRE_CLOSE(prob, textG.Probability(randomObs.unsafe_col(i)),
+          1e-8);
+      BOOST_REQUIRE_CLOSE(prob, binaryG.Probability(randomObs.unsafe_col(i)),
+          1e-8);
+    }
+  }
+}
+
+/**
+ * Laplace Distribution serialization test.
+ */
+BOOST_AUTO_TEST_CASE(LaplaceDistributionTest)
+{
+  arma::vec mean(20);
+  mean.randu();
+
+  LaplaceDistribution l(mean, 2.5);
+  LaplaceDistribution xmlL, textL, binaryL;
+
+  SerializeObjectAll(l, xmlL, textL, binaryL);
+
+  BOOST_REQUIRE_CLOSE(l.Scale(), xmlL.Scale(), 1e-8);
+  BOOST_REQUIRE_CLOSE(l.Scale(), textL.Scale(), 1e-8);
+  BOOST_REQUIRE_CLOSE(l.Scale(), binaryL.Scale(), 1e-8);
+
+  CheckMatrices(l.Mean(), xmlL.Mean(), textL.Mean(), binaryL.Mean());
+}
+
+/**
+ * Laplace Distribution Probability Test.
+ */
+BOOST_AUTO_TEST_CASE(LaplaceDistributionProbabilityTest)
+{
+  LaplaceDistribution l(arma::vec("0.0"), 1.0);
+
+  // Simple case.
+  BOOST_REQUIRE_CLOSE(l.Probability(arma::vec("0.0")),
+    0.500000000000000, 1e-5);
+  BOOST_REQUIRE_CLOSE(l.Probability(arma::vec("1.0")),
+    0.183939720585721, 1e-5);
+
+  arma::mat points = "0.0 1.0;";
+
+  arma::vec probabilities;
+
+  l.Probability(points, probabilities);
+
+  BOOST_REQUIRE_EQUAL(probabilities.n_elem, 2);
+
+  BOOST_REQUIRE_CLOSE(probabilities(0), 0.500000000000000, 1e-5);
+  BOOST_REQUIRE_CLOSE(probabilities(1), 0.183939720585721, 1e-5);
+}
+
+/**
+ * Laplace Distribution Log Probability Test.
+ */
+BOOST_AUTO_TEST_CASE(LaplaceDistributionLogProbabilityTest)
+{
+  LaplaceDistribution l(arma::vec("0.0"), 1.0);
+
+  // Simple case.
+  BOOST_REQUIRE_CLOSE(l.LogProbability(arma::vec("0.0")),
+    -0.693147180559945, 1e-5);
+  BOOST_REQUIRE_CLOSE(l.LogProbability(arma::vec("1.0")),
+    -1.693147180559946, 1e-5);
+
+  arma::mat points = "0.0 1.0;";
+
+  arma::vec logProbabilities;
+
+  l.LogProbability(points, logProbabilities);
+
+  BOOST_REQUIRE_EQUAL(logProbabilities.n_elem, 2);
+
+  BOOST_REQUIRE_CLOSE(logProbabilities(0), -0.693147180559945,
+    1e-5);
+  BOOST_REQUIRE_CLOSE(logProbabilities(1), -1.693147180559946,
+    1e-5);
+}
+
+/**
+ * Mahalanobis Distance serialization test.
+ */
+BOOST_AUTO_TEST_CASE(MahalanobisDistanceTest)
+{
+  MahalanobisDistance<> d;
+  d.Covariance().randu(50, 50);
+
+  MahalanobisDistance<> xmlD, textD, binaryD;
+
+  SerializeObjectAll(d, xmlD, textD, binaryD);
+
+  // Check the covariance matrices.
+  CheckMatrices(d.Covariance(),
+                xmlD.Covariance(),
+                textD.Covariance(),
+                binaryD.Covariance());
+}
+
+/**
+ * Regression distribution serialization test.
+ */
+BOOST_AUTO_TEST_CASE(RegressionDistributionTest)
+{
+  // Generate some random data.
+  arma::mat data;
+  data.randn(15, 800);
+  arma::rowvec responses;
+  responses.randn(800);
+
+  RegressionDistribution rd(data, responses);
+  RegressionDistribution xmlRd, textRd, binaryRd;
+
+  // Okay, now save it and load it.
+  SerializeObjectAll(rd, xmlRd, textRd, binaryRd);
+
+  // Check the gaussian distribution.
+  CheckMatrices(rd.Err().Mean(),
+                xmlRd.Err().Mean(),
+                textRd.Err().Mean(),
+                binaryRd.Err().Mean());
+  CheckMatrices(rd.Err().Covariance(),
+                xmlRd.Err().Covariance(),
+                textRd.Err().Covariance(),
+                binaryRd.Err().Covariance());
+
+  // Check the regression function.
+  if (rd.Rf().Lambda() == 0.0)
+  {
+    BOOST_REQUIRE_SMALL(xmlRd.Rf().Lambda(), 1e-8);
+    BOOST_REQUIRE_SMALL(textRd.Rf().Lambda(), 1e-8);
+    BOOST_REQUIRE_SMALL(binaryRd.Rf().Lambda(), 1e-8);
+  }
+  else
+  {
+    BOOST_REQUIRE_CLOSE(rd.Rf().Lambda(), xmlRd.Rf().Lambda(), 1e-8);
+    BOOST_REQUIRE_CLOSE(rd.Rf().Lambda(), textRd.Rf().Lambda(), 1e-8);
+    BOOST_REQUIRE_CLOSE(rd.Rf().Lambda(), binaryRd.Rf().Lambda(), 1e-8);
+  }
+
+  CheckMatrices(rd.Rf().Parameters(),
+                xmlRd.Rf().Parameters(),
+                textRd.Rf().Parameters(),
+                binaryRd.Rf().Parameters());
+}
+
+/*****************************************************/
+/** Diagonal Covariance Gaussian Distribution Tests **/
+/*****************************************************/
+
+/**
+ * Make sure Diagonal Covariance Gaussian distributions are initialized
+ * correctly.
+ */
+BOOST_AUTO_TEST_CASE(DiagonalGaussianDistributionEmptyConstructor)
+{
+  DiagonalGaussianDistribution d;
+
+  BOOST_REQUIRE_EQUAL(d.Mean().n_elem, 0);
+  BOOST_REQUIRE_EQUAL(d.Covariance().n_elem, 0);
+}
+
+/**
+ * Make sure Diagonal Covariance Gaussian distributions are initialized to
+ * the correct dimensionality.
+ */
+BOOST_AUTO_TEST_CASE(DiagonalGaussianDistributionDimensionalityConstructor)
+{
+  DiagonalGaussianDistribution d(4);
+
+  BOOST_REQUIRE_EQUAL(d.Mean().n_elem, 4);
+  BOOST_REQUIRE_EQUAL(d.Covariance().n_elem, 4);
+}
+
+/**
+ * Make sure Diagonal Covariance Gaussian distributions are initialized
+ * correctly when we give a mean and covariance.
+ */
+BOOST_AUTO_TEST_CASE(DiagonalGaussianDistributionConstructor)
+{
+  arma::vec mean = arma::randu<arma::vec>(3);
+  arma::vec covariance = arma::randu<arma::vec>(3);
+
+  DiagonalGaussianDistribution d(mean, covariance);
+
+  // Make sure the mean and covariance is correct.
+  for (size_t i = 0; i < 3; i++)
+  {
+    BOOST_REQUIRE_CLOSE(d.Mean()(i), mean(i), 1e-5);
+    BOOST_REQUIRE_CLOSE(d.Covariance()(i), covariance(i), 1e-5);
+  }
+}
+
+/**
+ * Make sure the probability of observations is correct.
+ * The values were calculated using 'dmvnorm' in R.
+ */
+BOOST_AUTO_TEST_CASE(DiagonalGaussianDistributionProbabilityTest)
+{
+  arma::vec mean("2 5 3 4 1");
+  arma::vec cov("3 1 5 3 2");
+
+  DiagonalGaussianDistribution d(mean, cov);
+
+  // Observations lists randomly selected.
+  BOOST_REQUIRE_CLOSE(d.LogProbability("3 5 2 7 8"), -20.861264167855161,
+      1e-5);
+  BOOST_REQUIRE_CLOSE(d.LogProbability("7 8 4 0 5"), -22.277930834521829,
+      1e-5);
+  BOOST_REQUIRE_CLOSE(d.LogProbability("6 8 7 7 5"), -21.111264167855161,
+      1e-5);
+  BOOST_REQUIRE_CLOSE(d.LogProbability("2 9 5 6 3"), -16.911264167855162,
+      1e-5);
+  BOOST_REQUIRE_CLOSE(d.LogProbability("5 8 2 9 7"), -26.111264167855161,
+      1e-5);
+}
+
+/**
+ * Test DiagonalGaussianDistribution::Probability() in the univariate case.
+ * The values were calculated using 'dmvnorm' in R.
+ */
+BOOST_AUTO_TEST_CASE(DiagonalGaussianUnivariateProbabilityTest)
+{
+  DiagonalGaussianDistribution d(arma::vec("0.0"), arma::vec("1.0"));
+
+  // Mean: 0.0, Covariance: 1.0
+  BOOST_REQUIRE_CLOSE(d.Probability("0.0"), 0.3989422804014327, 1e-5);
+  BOOST_REQUIRE_CLOSE(d.Probability("1.0"), 0.24197072451914337, 1e-5);
+  BOOST_REQUIRE_CLOSE(d.Probability("-1.0"), 0.24197072451914337, 1e-5);
+
+  // Mean: 0.0, Covariance: 2.0
+  d.Covariance("2.0");
+  BOOST_REQUIRE_CLOSE(d.Probability("0.0"), 0.28209479177387814, 1e-5);
+  BOOST_REQUIRE_CLOSE(d.Probability("1.0"), 0.21969564473386122, 1e-5);
+  BOOST_REQUIRE_CLOSE(d.Probability("-1.0"), 0.21969564473386122, 1e-5);
+
+  // Mean: 1.0, Covariance: 1.0
+  d.Mean() = "1.0";
+  d.Covariance("1.0");
+  BOOST_REQUIRE_CLOSE(d.Probability("0.0"), 0.24197072451914337, 1e-5);
+  BOOST_REQUIRE_CLOSE(d.Probability("1.0"), 0.3989422804014327, 1e-5);
+  BOOST_REQUIRE_CLOSE(d.Probability("-1.0"), 0.053990966513188056, 1e-5);
+
+  // Mean: 1.0, Covariance: 2.0
+  d.Covariance("2.0");
+  BOOST_REQUIRE_CLOSE(d.Probability("0.0"), 0.21969564473386122, 1e-5);
+  BOOST_REQUIRE_CLOSE(d.Probability("1.0"), 0.28209479177387814, 1e-5);
+  BOOST_REQUIRE_CLOSE(d.Probability("-1.0"), 0.10377687435514872, 1e-5);
+}
+
+/**
+ * Test DiagonalGaussianDistribution::Probability() in the multivariate case.
+ * The values were calculated using 'dmvnorm' in R.
+ */
+BOOST_AUTO_TEST_CASE(DiagonalGaussianMultivariateProbabilityTest)
+{
+  arma::vec mean("0 0");
+  arma::vec cov("2 2");
+  arma::vec obs("0 0");
+
+  DiagonalGaussianDistribution d(mean, cov);
+
+  BOOST_REQUIRE_CLOSE(d.Probability(obs), 0.079577471545947673, 1e-5);
+
+  obs = "1 1";
+  BOOST_REQUIRE_CLOSE(d.Probability(obs), 0.048266176315026957, 1e-5);
+
+  d.Mean() = "1 3";
+  BOOST_REQUIRE_CLOSE(d.Probability(obs), 0.029274915762159581, 1e-5);
+  BOOST_REQUIRE_CLOSE(d.Probability(-obs), 0.00053618878559782773, 1e-5);
+
+  // Higher dimensional case.
+  d.Mean() = "1 3 6 2 7";
+  d.Covariance("3 1 5 3 2");
+  obs = "2 5 7 3 8";
+  BOOST_REQUIRE_CLOSE(d.Probability(obs), 7.2790083003378082e-05, 1e-5);
+}
+
+/**
+ * Test the phi() function, for multiple points in the multivariate Gaussian
+ * case. The values were calculated using 'dmvnorm' in R.
+ */
+BOOST_AUTO_TEST_CASE(DiagonalGaussianMultipointMultivariateProbabilityTest)
+{
+  arma::vec mean = "2 5 3 7 2";
+  arma::vec cov("9 2 1 4 8");
+  arma::mat points = "3 5 2 7 5 8;"
+                     "2 6 8 3 4 6;"
+                     "1 4 2 7 8 2;"
+                     "6 8 4 7 9 2;"
+                     "4 6 7 7 3 2";
+  arma::vec phis;
+  DiagonalGaussianDistribution d(mean, cov);
+  d.LogProbability(points, phis);
+
+  BOOST_REQUIRE_EQUAL(phis.n_elem, 6);
+
+  BOOST_REQUIRE_CLOSE(phis(0), -12.453302051926864, 1e-5);
+  BOOST_REQUIRE_CLOSE(phis(1), -10.147746496371308, 1e-5);
+  BOOST_REQUIRE_CLOSE(phis(2), -13.210246496371308, 1e-5);
+  BOOST_REQUIRE_CLOSE(phis(3), -19.724135385260197, 1e-5);
+  BOOST_REQUIRE_CLOSE(phis(4), -21.585246496371308, 1e-5);
+  BOOST_REQUIRE_CLOSE(phis(5), -13.647746496371308, 1e-5);
+}
+
+/**
+ * Make sure random observations follow the probability distribution correctly.
+ */
+BOOST_AUTO_TEST_CASE(DiagonalGaussianDistributionRandomTest)
+{
+  arma::vec mean("2.5 1.25");
+  arma::vec cov("0.50 0.25");
+
+  DiagonalGaussianDistribution d(mean, cov);
+
+  arma::mat obs(2, 5000);
+
+  for (size_t i = 0; i < 5000; i++)
+    obs.col(i) = d.Random();
+
+  // Make sure that reflects the actual distribution.
+  arma::vec obsMean = arma::mean(obs, 1);
+  arma::mat obsCov = mlpack::math::ColumnCovariance(obs);
+
+  // 10% tolerance because this can be noisy.
+  BOOST_REQUIRE_CLOSE(obsMean(0), mean(0), 10.0);
+  BOOST_REQUIRE_CLOSE(obsMean(1), mean(1), 10.0);
+
+  BOOST_REQUIRE_CLOSE(obsCov(0, 0), cov(0), 10);
+  BOOST_REQUIRE_CLOSE(obsCov(1, 1), cov(1), 10);
+}
+
+/**
+ * Make sure that we can properly estimate from given observations.
+ */
+BOOST_AUTO_TEST_CASE(DiagonalGaussianDistributionTrainTest)
+{
+  arma::vec mean("2.5 1.5 8.2 3.1");
+  arma::vec cov("1.2 3.1 8.3 4.3");
+
+  // Generate the observations.
+  arma::mat observations(4, 10000);
+
+  for (size_t i = 0; i < 10000; i++)
+    observations.col(i) = (arma::sqrt(cov) % arma::randn<arma::vec>(4)) + mean;
+
+  DiagonalGaussianDistribution d;
+
+  // Calculate the actual mean and covariance of data using armadillo.
+  arma::vec actualMean = arma::mean(observations, 1);
+  arma::mat actualCov = mlpack::math::ColumnCovariance(observations);
+
+  // Estimate the parameters.
+  d.Train(observations);
+
+  // Check that the estimated parameters are right.
+  for (size_t i = 0; i < 4; i++)
+  {
+    BOOST_REQUIRE_SMALL(d.Mean()(i) - actualMean(i), 1e-5);
+    BOOST_REQUIRE_SMALL(d.Covariance()(i) - actualCov(i, i), 1e-5);
+  }
+}
+
+/**
+ * Make sure the unbiased estimator of the weighted sample works correctly.
+ * The values were calculated using 'cov.wt' in R.
+ */
+BOOST_AUTO_TEST_CASE(DiagonalGaussianUnbiasedEstimatorTest)
+{
+  // Generate the observations.
+  arma::mat observations("3 5 2 7;"
+                         "2 6 8 3;"
+                         "1 4 2 7;"
+                         "6 8 4 7");
+
+  arma::vec probs("0.3 0.4 0.1 0.2");
+
+  DiagonalGaussianDistribution d;
+
+  // Estimate the parameters.
+  d.Train(observations, probs);
+
+  BOOST_REQUIRE_CLOSE(d.Mean()(0), 4.5, 1e-5);
+  BOOST_REQUIRE_CLOSE(d.Mean()(1), 4.4, 1e-5);
+  BOOST_REQUIRE_CLOSE(d.Mean()(2), 3.5, 1e-5);
+  BOOST_REQUIRE_CLOSE(d.Mean()(3), 6.8, 1e-5);
+
+  BOOST_REQUIRE_CLOSE(d.Covariance()(0), 3.78571428571428603, 1e-5);
+  BOOST_REQUIRE_CLOSE(d.Covariance()(1), 6.34285714285714253, 1e-5);
+  BOOST_REQUIRE_CLOSE(d.Covariance()(2), 6.64285714285714235, 1e-5);
+  BOOST_REQUIRE_CLOSE(d.Covariance()(3), 2.22857142857142865, 1e-5);
+}
+
+/**
+ * Make sure that if all weights are the same, i.e. w_i / V1 = 1 / N, then
+ * the weighted mean and covariance reduce to the unweighted sample mean and
+ * covariance.
+ */
+BOOST_AUTO_TEST_CASE(DiagonalGaussianWeightedParametersReductionTest)
+{
+  arma::vec mean("2.5 1.5 8.2 3.1");
+  arma::vec cov("1.2 3.1 8.3 4.3");
+
+  // Generate the observations.
+  arma::mat obs(4, 5);
+  arma::vec probs("0.2 0.2 0.2 0.2 0.2");
+
+  for (size_t i = 0; i < 5; i++)
+    obs.col(i) = (arma::sqrt(cov) % arma::randn<arma::vec>(4)) + mean;
+
+  DiagonalGaussianDistribution d1;
+  DiagonalGaussianDistribution d2;
+
+  // Estimate the parameters.
+  d1.Train(obs);
+  d2.Train(obs, probs);
+
+  // Check if these are equal.
+  for (size_t i = 0; i < 4; i++)
+  {
+    BOOST_REQUIRE_CLOSE(d1.Mean()(i), d2.Mean()(i), 1e-5);
+    BOOST_REQUIRE_CLOSE(d1.Covariance()(i), d2.Covariance()(i), 1e-5);
+  }
 }
 
 BOOST_AUTO_TEST_SUITE_END();
