@@ -2,10 +2,10 @@
  * @file hmm_test_utils.hpp
  * @author Daivik Nema
  *
- * Structs for initializing and training HMMs (either of Discrete, Gaussian or
- * GMM HMMs). These structs are passed as template parameters to the
- * PerformAction function of an HMMModel object. These structs have been adapted
- * from the structs in mlpack/methods/hmm/hmm_train_main.cpp.
+ * Structs for initializing and training HMMs (either of Discrete, Gaussian,
+ * GMM, or Diagonal GMM HMMs). These structs are passed as template parameters
+ * to the PerformAction function of an HMMModel object. These structs have been
+ * adapted from the structs in mlpack/methods/hmm/hmm_train_main.cpp.
  *
  * mlpack is free software; you may redistribute it and/or modify it under the
  * terms of the 3-clause BSD license.  You should have received a copy of the
@@ -105,6 +105,33 @@ struct InitHMMModel
         tolerance);
   }
 
+  //! Helper function to create Diagonal GMM HMM.
+  static void Create(HMM<DiagonalGMM>& hmm,
+                     vector<mat>& trainSeq,
+                     size_t states,
+                     double tolerance = 1e-05)
+  {
+    // Find dimension of the data.
+    const size_t dimensionality = trainSeq[0].n_rows;
+    const int gaussians = 2;
+
+    if (gaussians == 0)
+    {
+      Log::Fatal << "Number of gaussians for each GMM must be specified "
+          << "when type = 'diag_gmm'!" << endl;
+    }
+
+    if (gaussians < 0)
+    {
+      Log::Fatal << "Invalid number of gaussians (" << gaussians << "); must "
+          << "be greater than or equal to 1." << endl;
+    }
+
+    // Create HMM object.
+    hmm = HMM<DiagonalGMM>(size_t(states), DiagonalGMM(size_t(gaussians),
+        dimensionality), tolerance);
+  }
+
   //! Helper function for discrete emission distributions.
   static void RandomInitialize(vector<DiscreteDistribution>& e)
   {
@@ -148,6 +175,28 @@ struct InitHMMModel
       }
     }
   }
+
+  //! Helper function for diagonal GMM emission distributions.
+  static void RandomInitialize(vector<DiagonalGMM>& e)
+  {
+    for (size_t i = 0; i < e.size(); ++i)
+    {
+      // Random weights.
+      e[i].Weights().randu();
+      e[i].Weights() /= arma::accu(e[i].Weights());
+
+      // Random means and covariances.
+      for (int g = 0; g < 2; ++g)
+      {
+        const size_t dimensionality = e[i].Component(g).Mean().n_rows;
+        e[i].Component(g).Mean().randu();
+
+        // Generate random diagonal covariance.
+        arma::vec r = arma::randu<arma::vec>(dimensionality);
+        e[i].Component(g).Covariance(r);
+      }
+    }
+  }
 };
 
 struct TrainHMMModel
