@@ -21,6 +21,7 @@
 #include <mlpack/methods/reinforcement_learning/environment/mountain_car.hpp>
 #include <mlpack/methods/reinforcement_learning/environment/acrobot.hpp>
 #include <mlpack/methods/reinforcement_learning/environment/cart_pole.hpp>
+#include <mlpack/methods/reinforcement_learning/environment/double_pole_cart.hpp>
 #include <mlpack/methods/reinforcement_learning/policy/greedy_policy.hpp>
 #include <mlpack/methods/reinforcement_learning/training_config.hpp>
 
@@ -102,6 +103,74 @@ BOOST_AUTO_TEST_CASE(CartPoleWithDQN)
   BOOST_REQUIRE(converged);
 }
 
+//! Test DQN in Cart Pole task with Prioritized Replay.
+BOOST_AUTO_TEST_CASE(CartPoleWithDQNPrioritizedReplay)
+{
+  // Set up the network.
+  FFN<MeanSquaredError<>, GaussianInitialization> model(MeanSquaredError<>(),
+      GaussianInitialization(0, 0.001));
+  model.Add<Linear<>>(4, 128);
+  model.Add<ReLULayer<>>();
+  model.Add<Linear<>>(128, 128);
+  model.Add<ReLULayer<>>();
+  model.Add<Linear<>>(128, 2);
+
+  // Set up the policy and replay method.
+  GreedyPolicy<CartPole> policy(1.0, 1000, 0.1);
+  PrioritizedReplay<CartPole> replayMethod(10, 10000, 0.6);
+
+  TrainingConfig config;
+  config.StepSize() = 0.01;
+  config.Discount() = 0.9;
+  config.TargetNetworkSyncInterval() = 100;
+  config.ExplorationSteps() = 100;
+  config.DoubleQLearning() = false;
+  config.StepLimit() = 200;
+
+  // Set up DQN agent.
+  QLearning<CartPole, decltype(model), AdamUpdate, decltype(policy),
+      decltype(replayMethod)>
+      agent(std::move(config), std::move(model), std::move(policy),
+          std::move(replayMethod));
+
+  arma::running_stat<double> averageReturn;
+  size_t episodes = 0;
+  bool converged = true;
+  while (true)
+  {
+    double episodeReturn = agent.Episode();
+    averageReturn(episodeReturn);
+    episodes += 1;
+
+    if (episodes > 1000)
+    {
+      Log::Debug << "Cart Pole with DQN failed." << std::endl;
+      converged = false;
+      break;
+    }
+
+    /**
+     * Reaching running average return 35 is enough to show it works.
+     * For the speed of the test case, I didn't set high criterion.
+     */
+    Log::Debug << "Average return: " << averageReturn.mean()
+        << " Episode return: " << episodeReturn << std::endl;
+    if (averageReturn.mean() > 35)
+    {
+      agent.Deterministic() = true;
+      arma::running_stat<double> testReturn;
+      for (size_t i = 0; i < 10; ++i)
+        testReturn(agent.Episode());
+
+      Log::Debug << "Average return in deterministic test: "
+          << testReturn.mean() << std::endl;
+      break;
+    }
+  }
+
+  BOOST_REQUIRE(converged);
+}
+
 //! Test Double DQN in Cart Pole task.
 BOOST_AUTO_TEST_CASE(CartPoleWithDoubleDQN)
 {
@@ -328,4 +397,74 @@ BOOST_AUTO_TEST_CASE(MountainCarWithDQN)
   BOOST_REQUIRE_EQUAL(success, true);
 }
 
+//! Test DQN in DoublePoleCart task.
+BOOST_AUTO_TEST_CASE(DoublePoleCartWithDQN)
+{
+  // We will allow four trials total.
+  bool success = false;
+  for (size_t trial = 0; trial < 4; trial++)
+  {
+    // Set up the network.
+    FFN<MeanSquaredError<>, GaussianInitialization> model(MeanSquaredError<>(),
+        GaussianInitialization(0, 0.001));
+    model.Add<Linear<>>(6, 256);
+    model.Add<ReLULayer<>>();
+    model.Add<Linear<>>(256, 3);
+
+    // Set up the policy and replay method.
+    GreedyPolicy<DoublePoleCart> policy(1.0, 1000, 0.1, 0.99);
+    RandomReplay<DoublePoleCart> replayMethod(20, 10000);
+
+    TrainingConfig config;
+    config.StepSize() = 0.01;
+    config.Discount() = 0.9;
+    config.TargetNetworkSyncInterval() = 100;
+    config.ExplorationSteps() = 100;
+    config.DoubleQLearning() = false;
+    config.StepLimit() = 600;
+
+    // Set up DQN agent.
+    QLearning<DoublePoleCart, decltype(model), AdamUpdate, decltype(policy)>
+        agent(std::move(config), std::move(model), std::move(policy),
+        std::move(replayMethod));
+
+    size_t episodes = 0;
+    bool converged = true;
+    size_t episodeSuccesses = 0;
+    while (true)
+    {
+      double episodeReturn = agent.Episode();
+      episodes += 1;
+
+      if (episodeReturn >= 280)
+        episodeSuccesses++;
+
+      if (episodes > 2000)
+      {
+        Log::Debug << "Cart Pole with DQN failed." << std::endl;
+        converged = false;
+        break;
+      }
+
+      // If the model can solve the environment in two trials this is fine for
+      // a simple test.
+      if (episodeSuccesses >= 2)
+      {
+        Log::Debug << "QLearning has succeeded in the multiple pole cart" <<
+            " environment." << std::endl;
+        converged = true;
+        break;
+      }
+    }
+
+    if (converged)
+    {
+      success = true;
+      break;
+    }
+  }
+
+  BOOST_REQUIRE_EQUAL(success, true);
+}
+
 BOOST_AUTO_TEST_SUITE_END();
