@@ -11,11 +11,11 @@
  */
 #include <mlpack/core.hpp>
 
-#include <mlpack/core/optimizers/sgd/sgd.hpp>
-#include <mlpack/core/optimizers/rmsprop/rmsprop.hpp>
+#include <ensmallen.hpp>
 #include <mlpack/methods/ann/layer/layer.hpp>
 #include <mlpack/methods/ann/loss_functions/mean_squared_error.hpp>
 #include <mlpack/methods/ann/rnn.hpp>
+#include <mlpack/methods/ann/brnn.hpp>
 #include <mlpack/core/data/binarize.hpp>
 #include <mlpack/core/math/random.hpp>
 
@@ -26,7 +26,7 @@
 
 using namespace mlpack;
 using namespace mlpack::ann;
-using namespace mlpack::optimization;
+using namespace ens;
 using namespace mlpack::math;
 
 BOOST_AUTO_TEST_SUITE(RecurrentNetworkTest);
@@ -72,6 +72,78 @@ void GenerateNoisySines(arma::cube& data,
   }
 }
 
+/**
+ * Train the BRNN on a larger dataset.
+ */
+BOOST_AUTO_TEST_CASE(SequenceClassificationBRNNTest)
+{
+  // Using same test for RNN below.
+  size_t successes = 0;
+  const size_t rho = 10;
+
+  for (size_t trial = 0; trial < 6; ++trial)
+  {
+    // Generate 12 (2 * 6) noisy sines. A single sine contains rho
+    // points/features.
+    arma::cube input;
+    arma::mat labelsTemp;
+    GenerateNoisySines(input, labelsTemp, rho, 6);
+
+    arma::cube labels = arma::zeros<arma::cube>(1, labelsTemp.n_cols, rho);
+    for (size_t i = 0; i < labelsTemp.n_cols; ++i)
+    {
+      const int value = arma::as_scalar(arma::find(
+          arma::max(labelsTemp.col(i)) == labelsTemp.col(i), 1)) + 1;
+      labels.tube(0, i).fill(value);
+    }
+
+    Add<> add(4);
+    Linear<> lookup(1, 4);
+    SigmoidLayer<> sigmoidLayer;
+    Linear<> linear(4, 4);
+    Recurrent<>* recurrent = new Recurrent<>(
+        add, lookup, linear, sigmoidLayer, rho);
+
+    BRNN<> model(rho);
+    model.Add<IdentityLayer<> >();
+    model.Add(recurrent);
+    model.Add<Linear<> >(4, 5);
+
+    StandardSGD opt(0.1, 1, 500 * input.n_cols, -100);
+    model.Train(input, labels, opt);
+    BOOST_TEST_CHECKPOINT("Training over");
+    arma::cube prediction;
+    model.Predict(input, prediction);
+    BOOST_TEST_CHECKPOINT("Predicion over");
+
+    size_t error = 0;
+    for (size_t i = 0; i < prediction.n_cols; ++i)
+    {
+      const int predictionValue = arma::as_scalar(arma::find(
+          arma::max(prediction.slice(rho - 1).col(i)) ==
+          prediction.slice(rho - 1).col(i), 1) + 1);
+
+      const int targetValue = arma::as_scalar(arma::find(
+          arma::max(labelsTemp.col(i)) == labelsTemp.col(i), 1)) + 1;
+
+      if (predictionValue == targetValue)
+      {
+        error++;
+      }
+    }
+
+    double classificationError = 1 - double(error) / prediction.n_cols;
+    BOOST_TEST_CHECKPOINT(classificationError);
+    if (classificationError <= 0.2)
+    {
+      ++successes;
+      break;
+    }
+  }
+
+  BOOST_REQUIRE_GE(successes, 1);
+}
+
 /**
  * Train the vanilla network on a larger dataset.
  */
@@ -152,7 +224,6 @@ BOOST_AUTO_TEST_CASE(SequenceClassificationTest)
     }
 
     double classificationError = 1 - double(error) / prediction.n_cols;
-
     if (classificationError <= 0.2)
     {
       ++successes;
@@ -448,12 +519,13 @@ arma::Mat<char> GenerateReberGrammarData(
 /**
  * Train the specified network and the construct a Reber grammar dataset.
  */
-template<typename RecurrentLayerType>
-void ReberGrammarTestNetwork(const size_t hiddenSize = 4,
+template<typename ModelType>
+void ReberGrammarTestNetwork(ModelType& model,
                              const bool recursive = false,
                              const size_t averageRecursion = 3,
                              const size_t maxRecursion = 5,
-                             const size_t iterations = 10)
+                             const size_t iterations = 10,
+                             const size_t trials = 5)
 {
   const size_t trainReberGrammarCount = 700;
   const size_t testReberGrammarCount = 250;
@@ -494,16 +566,11 @@ void ReberGrammarTestNetwork(const size_t hiddenSize = 4,
   // to escape from local minima and to solve the task.
   size_t successes = 0;
   size_t offset = 0;
-  for (size_t trial = 0; trial < 5; ++trial)
+  const size_t inputSize = 7;
+  for (size_t trial = 0; trial < trials; ++trial)
   {
-    const size_t outputSize = 7;
-    const size_t inputSize = 7;
-
-    RNN<MeanSquaredError<> > model(5);
-    model.Add<Linear<> >(inputSize, hiddenSize);
-    model.Add<RecurrentLayerType>(hiddenSize, hiddenSize);
-    model.Add<Linear<> >(hiddenSize, outputSize);
-    model.Add<SigmoidLayer<> >();
+    // Reset model before using for next trial.
+    model.Reset();
     MomentumSGD opt(0.06, 50, 2, -50000);
 
     arma::cube inputTemp, labelsTemp;
@@ -586,7 +653,12 @@ void ReberGrammarTestNetwork(const size_t hiddenSize = 4,
  */
 BOOST_AUTO_TEST_CASE(LSTMReberGrammarTest)
 {
-  ReberGrammarTestNetwork<LSTM<> >(10, false);
+  RNN<MeanSquaredError<> > model(5);
+  model.Add<Linear<> >(7, 10);
+  model.Add<LSTM<> >(10, 10);
+  model.Add<Linear<> >(10, 7);
+  model.Add<SigmoidLayer<> >();
+  ReberGrammarTestNetwork(model, false);
 }
 
 /**
@@ -594,7 +666,12 @@ BOOST_AUTO_TEST_CASE(LSTMReberGrammarTest)
  */
 BOOST_AUTO_TEST_CASE(FastLSTMReberGrammarTest)
 {
-  ReberGrammarTestNetwork<FastLSTM<> >(8, false);
+  RNN<MeanSquaredError<> > model(5);
+  model.Add<Linear<> >(7, 8);
+  model.Add<FastLSTM<> >(8, 8);
+  model.Add<Linear<> >(8, 7);
+  model.Add<SigmoidLayer<> >();
+  ReberGrammarTestNetwork(model, false);
 }
 
 /**
@@ -602,7 +679,24 @@ BOOST_AUTO_TEST_CASE(FastLSTMReberGrammarTest)
  */
 BOOST_AUTO_TEST_CASE(GRURecursiveReberGrammarTest)
 {
-  ReberGrammarTestNetwork<GRU<> >(16, true);
+  RNN<MeanSquaredError<> > model(5);
+  model.Add<Linear<> >(7, 16);
+  model.Add<GRU<> >(16, 16);
+  model.Add<Linear<> >(16, 7);
+  model.Add<SigmoidLayer<> >();
+  ReberGrammarTestNetwork(model, true, 3, 5, 10, 7);
+}
+
+/**
+ * Train BLSTM on an embedded Reber grammar dataset.
+ */
+BOOST_AUTO_TEST_CASE(BRNNReberGrammarTest)
+{
+  BRNN<MeanSquaredError<>, AddMerge<>, SigmoidLayer<> > model(5);
+  model.Add<Linear<> >(7, 10);
+  model.Add<LSTM<> >(10, 10);
+  model.Add<Linear<> >(10, 7);
+  ReberGrammarTestNetwork(model, false, 3, 5, 1);
 }
 
 /*
@@ -989,12 +1083,12 @@ void ReberGrammarTestCustomNetwork(const size_t hiddenSize = 4,
    *            .......
    */
   // It isn't guaranteed that the recurrent network will converge in the
-  // specified number of iterations using random weights. If this works 1 of 5
+  // specified number of iterations using random weights. If this works 1 of 10
   // times, I'm fine with that. All I want to know is that the network is able
   // to escape from local minima and to solve the task.
   size_t successes = 0;
   size_t offset = 0;
-  for (size_t trial = 0; trial < 5; ++trial)
+  for (size_t trial = 0; trial < 10; ++trial)
   {
     const size_t outputSize = 7;
     const size_t inputSize = 7;
@@ -1069,7 +1163,7 @@ void ReberGrammarTestCustomNetwork(const size_t hiddenSize = 4,
     }
 
     error /= testReberGrammarCount;
-    if (error <= 0.3)
+    if (error <= 0.35)
     {
       ++successes;
       break;
@@ -1224,4 +1318,99 @@ BOOST_AUTO_TEST_CASE(MultiTimestepTest)
   BOOST_REQUIRE_LE(err, 0.025);
 }
 
+/**
+ * Test that RNN::Train() returns finite objective value.
+ */
+BOOST_AUTO_TEST_CASE(RNNTrainReturnObjective)
+{
+  const size_t rho = 10;
+
+  // Generate 12 (2 * 6) noisy sines. A single sine contains rho
+  // points/features.
+  arma::cube input;
+  arma::mat labelsTemp;
+  GenerateNoisySines(input, labelsTemp, rho, 6);
+
+  arma::cube labels = arma::zeros<arma::cube>(1, labelsTemp.n_cols, rho);
+  for (size_t i = 0; i < labelsTemp.n_cols; ++i)
+  {
+    const int value = arma::as_scalar(arma::find(
+        arma::max(labelsTemp.col(i)) == labelsTemp.col(i), 1)) + 1;
+    labels.tube(0, i).fill(value);
+  }
+
+  /**
+   * Construct a network with 1 input unit, 4 hidden units and 10 output
+   * units. The hidden layer is connected to itself. The network structure
+   * looks like:
+   *
+   *  Input         Hidden        Output
+   * Layer(1)      Layer(4)      Layer(10)
+   * +-----+       +-----+       +-----+
+   * |     |       |     |       |     |
+   * |     +------>|     +------>|     |
+   * |     |    ..>|     |       |     |
+   * +-----+    .  +--+--+       +-----+
+   *            .     .
+   *            .     .
+   *            .......
+   */
+  Add<> add(4);
+  Linear<> lookup(1, 4);
+  SigmoidLayer<> sigmoidLayer;
+  Linear<> linear(4, 4);
+  Recurrent<>* recurrent = new Recurrent<>(add, lookup, linear,
+      sigmoidLayer, rho);
+
+  RNN<> model(rho);
+  model.Add<IdentityLayer<> >();
+  model.Add(recurrent);
+  model.Add<Linear<> >(4, 10);
+  model.Add<LogSoftMax<> >();
+
+  StandardSGD opt(0.1, 1, input.n_cols /* 1 epoch */, -100);
+  double objVal = model.Train(input, labels, opt);
+
+  BOOST_REQUIRE_EQUAL(std::isfinite(objVal), true);
+}
+
+/**
+ * Test that BRNN::Train() returns finite objective value.
+ */
+BOOST_AUTO_TEST_CASE(BRNNTrainReturnObjective)
+{
+  const size_t rho = 10;
+
+  arma::cube input;
+  arma::mat labelsTemp;
+  GenerateNoisySines(input, labelsTemp, rho, 6);
+
+  arma::cube labels = arma::zeros<arma::cube>(1, labelsTemp.n_cols, rho);
+  for (size_t i = 0; i < labelsTemp.n_cols; ++i)
+  {
+    const int value = arma::as_scalar(arma::find(
+        arma::max(labelsTemp.col(i)) == labelsTemp.col(i), 1)) + 1;
+    labels.tube(0, i).fill(value);
+  }
+
+  Add<> add(4);
+  Linear<> lookup(1, 4);
+  SigmoidLayer<> sigmoidLayer;
+  Linear<> linear(4, 4);
+  Recurrent<>* recurrent = new Recurrent<>(
+      add, lookup, linear, sigmoidLayer, rho);
+
+  BRNN<> model(rho);
+  model.Add<IdentityLayer<> >();
+  model.Add(recurrent);
+  model.Add<Linear<> >(4, 5);
+
+  StandardSGD opt(0.1, 1, 500 * input.n_cols, -100);
+  double objVal = model.Train(input, labels, opt);
+  BOOST_TEST_CHECKPOINT("Training over");
+
+  // Test that BRNN::Train() returns finite objective value.
+  BOOST_REQUIRE_EQUAL(std::isfinite(objVal), true);
+}
+
 BOOST_AUTO_TEST_SUITE_END();
