@@ -16,6 +16,8 @@
 #include <mlpack/methods/reinforcement_learning/environment/mountain_car.hpp>
 #include <mlpack/methods/reinforcement_learning/environment/continuous_mountain_car.hpp>
 #include <mlpack/methods/reinforcement_learning/environment/cart_pole.hpp>
+#include <mlpack/methods/reinforcement_learning/environment/double_pole_cart.hpp>
+#include <mlpack/methods/reinforcement_learning/environment/continuous_double_pole_cart.hpp>
 #include <mlpack/methods/reinforcement_learning/environment/acrobot.hpp>
 #include <mlpack/methods/reinforcement_learning/environment/pendulum.hpp>
 #include <mlpack/methods/reinforcement_learning/replay/random_replay.hpp>
@@ -35,7 +37,8 @@ BOOST_AUTO_TEST_SUITE(RLComponentsTest)
  */
 BOOST_AUTO_TEST_CASE(SimplePendulumTest)
 {
-  const Pendulum task = Pendulum();
+  Pendulum task = Pendulum();
+  task.MaxSteps() = 5;
 
   Pendulum::State state = task.InitialSample();
   Pendulum::Action action;
@@ -45,6 +48,16 @@ BOOST_AUTO_TEST_CASE(SimplePendulumTest)
   // The reward is always negative. Check if not lower than lowest possible.
   BOOST_REQUIRE(reward >= -(pow(M_PI, 2) + 6.404));
 
+  BOOST_REQUIRE(!task.IsTerminal(state));
+
+  while (!task.IsTerminal(state))
+    task.Sample(state, action, state);
+
+  // Check if the number of steps performed is less or equal as the maximum
+  // allowed, since we use a random action there is no guarantee that we will
+  // reach the maximum number of steps.
+  BOOST_REQUIRE_LE(task.StepsPerformed(), 5);
+
   // The action is simply the torque. Check if dimension is 1.
   BOOST_REQUIRE_EQUAL(1, action.size);
 }
@@ -55,7 +68,8 @@ BOOST_AUTO_TEST_CASE(SimplePendulumTest)
  */
 BOOST_AUTO_TEST_CASE(SimpleContinuousMountainCarTest)
 {
-  const ContinuousMountainCar task = ContinuousMountainCar();
+  ContinuousMountainCar task = ContinuousMountainCar();
+  task.MaxSteps() = 5;
 
   ContinuousMountainCar::State state = task.InitialSample();
   ContinuousMountainCar::Action action;
@@ -64,6 +78,14 @@ BOOST_AUTO_TEST_CASE(SimpleContinuousMountainCarTest)
   // Maximum reward possible is 100.
   BOOST_REQUIRE(reward <= 100.0);
   BOOST_REQUIRE(!task.IsTerminal(state));
+
+  while (!task.IsTerminal(state))
+    task.Sample(state, action, state);
+
+  // Check if the number of steps performed is the same as the maximum allowed.
+  BOOST_REQUIRE_EQUAL(task.StepsPerformed(), 5);
+
+  // Check if the size of the action space is 1.
   BOOST_REQUIRE_EQUAL(1, action.size);
 }
 
@@ -73,7 +95,8 @@ BOOST_AUTO_TEST_CASE(SimpleContinuousMountainCarTest)
  */
 BOOST_AUTO_TEST_CASE(SimpleAcrobotTest)
 {
-  const Acrobot task = Acrobot();
+  Acrobot task = Acrobot();
+  task.MaxSteps() = 5;
 
   Acrobot::State state = task.InitialSample();
   Acrobot::Action action = Acrobot::Action::negativeTorque;
@@ -81,6 +104,14 @@ BOOST_AUTO_TEST_CASE(SimpleAcrobotTest)
 
   BOOST_REQUIRE_EQUAL(reward, -1.0);
   BOOST_REQUIRE(!task.IsTerminal(state));
+
+  while (!task.IsTerminal(state))
+    task.Sample(state, action, state);
+
+  // Check if the number of steps performed is the same as the maximum allowed.
+  BOOST_REQUIRE_EQUAL(task.StepsPerformed(), 5);
+
+  // Check if the size of the action space is 3.
   BOOST_REQUIRE_EQUAL(3, Acrobot::Action::size);
 }
 
@@ -90,7 +121,8 @@ BOOST_AUTO_TEST_CASE(SimpleAcrobotTest)
  */
 BOOST_AUTO_TEST_CASE(SimpleMountainCarTest)
 {
-  const MountainCar task = MountainCar();
+  MountainCar task = MountainCar();
+  task.MaxSteps() = 5;
 
   MountainCar::State state = task.InitialSample();
   MountainCar::Action action = MountainCar::Action::backward;
@@ -98,16 +130,25 @@ BOOST_AUTO_TEST_CASE(SimpleMountainCarTest)
 
   BOOST_REQUIRE_EQUAL(reward, -1.0);
   BOOST_REQUIRE(!task.IsTerminal(state));
+
+  while (!task.IsTerminal(state))
+    task.Sample(state, action, state);
+
+  // Check if the number of steps performed is the same as the maximum allowed.
+  BOOST_REQUIRE_EQUAL(task.StepsPerformed(), 5);
+
+  // Check if the size of the action space is 3.
   BOOST_REQUIRE_EQUAL(3, MountainCar::Action::size);
 }
 
 /**
- * Constructs a CartPole instance and check if the main rountine works as
+ * Constructs a CartPole instance and check if the main routine works as
  * it should be.
  */
 BOOST_AUTO_TEST_CASE(SimpleCartPoleTest)
 {
-  const CartPole task = CartPole();
+  CartPole task = CartPole();
+  task.MaxSteps() = 5;
 
   CartPole::State state = task.InitialSample();
   CartPole::Action action = CartPole::Action::backward;
@@ -115,9 +156,65 @@ BOOST_AUTO_TEST_CASE(SimpleCartPoleTest)
 
   BOOST_REQUIRE_EQUAL(reward, 1.0);
   BOOST_REQUIRE(!task.IsTerminal(state));
+
+  while (!task.IsTerminal(state))
+    task.Sample(state, action, state);
+
+  // Check if the number of steps performed is the same as the maximum allowed.
+  BOOST_REQUIRE_EQUAL(task.StepsPerformed(), 5);
+
   BOOST_REQUIRE_EQUAL(2, CartPole::Action::size);
 }
 
+/**
+ * Constructs a DoublePoleCart instance and check if the main routine works as
+ * it should be.
+ */
+BOOST_AUTO_TEST_CASE(DoublePoleCartTest)
+{
+  DoublePoleCart task = DoublePoleCart();
+  task.MaxSteps() = 5;
+
+  DoublePoleCart::State state = task.InitialSample();
+  DoublePoleCart::Action action = DoublePoleCart::Action::backward;
+  double reward = task.Sample(state, action);
+
+  BOOST_REQUIRE_EQUAL(reward, 1.0);
+  BOOST_REQUIRE(!task.IsTerminal(state));
+
+  while (!task.IsTerminal(state))
+    task.Sample(state, action, state);
+
+  // Check if the number of steps performed is the same as the maximum allowed.
+  BOOST_REQUIRE_EQUAL(task.StepsPerformed(), 5);
+  BOOST_REQUIRE_EQUAL(2, DoublePoleCart::Action::size);
+}
+
+/**
+ * Constructs a ContinuousDoublePoleCart instance and check if the main 
+ * routine works as it should be.
+ */
+BOOST_AUTO_TEST_CASE(ContinuousDoublePoleCartTest)
+{
+  ContinuousDoublePoleCart task = ContinuousDoublePoleCart();
+  task.MaxSteps() = 5;
+
+  ContinuousDoublePoleCart::State state = task.InitialSample();
+  ContinuousDoublePoleCart::Action action;
+  action.action[0] = math::Random(-1.0, 1.0);
+  double reward = task.Sample(state, action);
+
+  BOOST_REQUIRE_EQUAL(reward, 1.0);
+  BOOST_REQUIRE(!task.IsTerminal(state));
+
+  while (!task.IsTerminal(state))
+    task.Sample(state, action, state);
+
+  // Check if the number of steps performed is the same as the maximum allowed.
+  BOOST_REQUIRE_EQUAL(task.StepsPerformed(), 5);
+  BOOST_REQUIRE_EQUAL(1, action.size);
+}
+
 /**
  * Construct a random replay instance and check if it works as
  * it should be.
